************************************************************************
        function A0_(m)                            
************************************************************************
*	scalar 1-point function
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 A0_

	COMMON/UV/MUDIM2

	A0_ = 0d0
	if (m.ne.0d0) A0_ = m**2*(1d0+log(mudim2/m**2))

	end

************************************************************************
        function B0_(p,m0,m1,ext)                            
************************************************************************
*	scalar 2-point function
*-----------------------------------------------------------------------
*	General result from A.Denner, Fortschr. Phys. 41 (1993) 307
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 B0_,B0_ext,ieps,r,sqe
	integer ext

	COMMON/UV/MUDIM2

	if (ext.ne.0) then
	  B0_ = B0_ext(p,m0,m1,ext)
	  return
	endif

	eps  = 1d-20
	ieps = dcmplx(0d0,eps)

	m02 = m0**2
	m12 = m1**2

c*** cases with p=0
	if (p.eq.0d0) then
	  if (m0.eq.0d0) then
	  B0_ = 1d0+log(mudim2/m12)
	  elseif (m1.eq.0d0) then
	  B0_ = 1d0+log(mudim2/m02)
	  elseif (m0.eq.m1) then
	  B0_ = log(mudim2/m02)
	  else
	  B0_ = 1d0-(m02*log(m02/mudim2)-m12*log(m12/mudim2))/(m02-m12)
	  endif
c*** cases with p=/=0 and mi=0
	elseif (m0*m1.eq.0d0) then
	  if ((m0.eq.0d0).and.(m1.eq.0d0)) then
	  B0_ = 2d0-log(-p/mudim2-ieps)
	  elseif (p.eq.m02) then
	  B0_ = 2d0+log(mudim2/m02)
	  elseif (p.eq.m12) then
	  B0_ = 2d0+log(mudim2/m12)
	  elseif (m0.eq.0d0) then
	  B0_ = 2d0-log(m12/mudim2)+(m12/p-1d0)*log(1d0-p/m12-ieps)
	  elseif (m1.eq.0d0) then
	  B0_ = 2d0-log(m02/mudim2)+(m02/p-1d0)*log(1d0-p/m02-ieps)
	  endif
c*** general case
	else
	  r = -sqe( dcmplx(1d0),(m02+m12-p)/m0/m1-ieps,dcmplx(1d0) )
	  B0_ = 2d0-log(m0*m1/mudim2)+(m02-m12)/p*log(m1/m0)
     &		-m0*m1/p*(1d0/r-r)*log(r)
	endif

	end

************************************************************************
        function DB0_(p,m0,m1,ext)                            
************************************************************************
*	momentum derivative of scalar 2-point function
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 DB0_,DB0_ext,ieps,r,sqe
	integer ext

	COMMON/IR/LAMBDA,LAMBDA2

	if (ext.ne.0) then
	  DB0_ = DB0_ext(p,m0,m1,ext)
	  return
	endif

	eps  = 1d-20
	ieps = dcmplx(0d0,eps)

	m02 = m0**2
	m12 = m1**2

c*** cases with p=0
	if (p.eq.0d0) then
	  if (m0.eq.0d0) then
	  DB0_ = 1d0/2d0/m12
	  elseif (m1.eq.0d0) then
	  DB0_ = 1d0/2d0/m02
	  elseif (m0.eq.m1) then
	  DB0_ = 1d0/6d0/m02
	  else
	  DB0_ = (m02**2-m12**2-2d0*m02*m12*log(m02/m12))
     &		  /2d0/(m02-m12)**3
	  endif
c*** cases with p=/=0 and mi=0
	elseif (m0*m1.eq.0d0) then
	  if ((m0.eq.0d0).and.(m1.eq.0d0)) then
	  DB0_ = -1d0/p
	  elseif (p.eq.m02) then
	  DB0_ = -1d0/2d0/m02*(2d0+log(lambda2/m02))
	  elseif (p.eq.m12) then
	  DB0_ = -1d0/2d0/m12*(2d0+log(lambda2/m12))
	  elseif (m0.eq.0d0) then
	  DB0_ = -1d0/p**2*(p+m12*log(1d0-p/m12-ieps))
	  elseif (m1.eq.0d0) then
	  DB0_ = -1d0/p**2*(p+m02*log(1d0-p/m02-ieps))
	  endif
c*** general case
	else
	  r = -sqe( dcmplx(1d0),(m02+m12-p)/m0/m1-ieps,dcmplx(1d0) )
	  DB0_ = (m02-m12)/p**2*log(m0/m1)+m0*m1/p**2*(1d0/r-r)*log(r)
     &		 -1d0/p*(1d0+(r**2+1d0)/(r**2-1d0)*log(r))
	endif

	end

************************************************************************
        function B1_(p,m0,m1,ext)                            
************************************************************************
*	coefficient for vector 2-point function
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 B1_,B1_ext,B0_,DB0_,A0_
	integer ext

	if (ext.ne.0) then
	  B1_ = B1_ext(p,m0,m1,ext)
	  return
	endif

	m02 = m0**2
	m12 = m1**2

	if (p.eq.0d0) then
	  B1_ = -B0_(0d0,m0,m1,ext)/2d0
     &		+(m12-m02)/2d0*DB0_(0d0,m0,m1,ext)
	else
	  B1_ = ( A0_(m0)-A0_(m1)-(p-m12+m02)*B0_(p,m0,m1,ext) )/2d0/p
	endif

	end

************************************************************************
        function DB1_(p,m0,m1,ext)                            
************************************************************************
*	momentum derivative of coefficient for vector 2-point function
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 DB1_,DB1_ext,B1_,B0_,DB0_,A0_
	integer ext

	if (ext.ne.0) then
	  DB1_ = DB1_ext(p,m0,m1,ext)
	  return
	endif

	m02 = m0**2
	m12 = m1**2
	m04 = m0**4
	m14 = m1**4

	if (p.eq.0d0) then
	  if (m0.eq.0d0) then
	  DB1_ = -1d0/6d0/m12
	  elseif (m1.eq.0d0) then
	  DB1_ = -1d0/3d0/m02
	  elseif (m0.eq.m1) then
	  DB1_ = -1d0/12d0/m02
	  else
	  DB1_ = ( 2d0*m04+5d0*m02*m12-m14
     &		  -6d0*m02*(A0_(m1)*m02-A0_(m0)*m12)/(m02-m12))
     &		 /6d0/(m12-m02)**3
	  endif
	else
	  DB1_ = -( 2d0*B1_(p,m0,m1,ext)+B0_(p,m0,m1,ext)
     &		   +(p-m12+m02)*DB0_(p,m0,m1,ext) )/2d0/p
	endif

	end

************************************************************************
        function B20_(p,m0,m1,ext)                            
************************************************************************
*	tensor coefficient B00 for 2nd-rank tensor 2-point function
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 B20_,B20_ext,B1_,B0_,A0_
	integer ext

	if (ext.ne.0) then
	  B20_ = B20_ext(p,m0,m1,ext)
	  return
	endif

	m02 = m0**2
	m12 = m1**2

	if (p.eq.0d0) then
	  if (m0.eq.m1) then
	  B20_ = A0_(m1)/2d0
	  else
	  B20_ = (m02+m12)/8D0+(A0_(m0)*m02-A0_(m1)*m12)/4D0/(m02-m12)
	  endif
	else
	  B20_ = ( A0_(m1)+2d0*m02*B0_(p,m0,m1,ext)
     &		   +(p-m12+m02)*B1_(p,m0,m1,ext)+m02+m12-p/3d0 )/6d0
	endif

	end

************************************************************************
        function B21_(p,m0,m1,ext)                            
************************************************************************
*	tensor coefficient B11 for 2nd-rank tensor 2-point function
*-----------------------------------------------------------------------
*	30.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 B21_,B21_ext,B1_,B0_,A0_
	integer ext

	if (ext.ne.0) then
	  B21_ = B21_ext(p,m0,m1,ext)
	  return
	endif

	m02 = m0**2
	m12 = m1**2
	m04 = m0**4
	m14 = m1**4

	if (p.eq.0d0) then
	  if (m0.eq.m1) then
	  B21_ = B0_(0d0,m0,m1,ext)/3d0
	  else
	  B21_ = ( 5d0*m04+5d0*m02*m12-4d0*m14
     &		   +6d0/(m02-m12)*A0_(m0)*m04
     &		   -6d0/(m02-m12)*A0_(m1)*(3d0*m04-3d0*m02*m12+m14) )
     &		  /18d0/(m02-m12)**2
	  endif
	else
	  B21_ = ( 2d0*A0_(m1)-2d0*m02*B0_(p,m0,m1,ext)
     &		 -4d0*(p-m12+m02)*B1_(p,m0,m1,ext)-m02-m12+p/3d0 )/6d0/p
	endif

	end

************************************************************************
        function C0_(q01,q12,q20,m0,m1,m2,ext)                            
************************************************************************
*	scalar 3-point function
*-----------------------------------------------------------------------
*	General result from A.Denner, Fortschr. Phys. 41 (1993) 307
*-----------------------------------------------------------------------
*	29.6.98 Stefan Dittmaier
************************************************************************
        implicit real*8 (a-z)                                         
	complex*16 c0_,C0_ext,cspen,eta,ieps,alpha,alp(0:2),sc,xs
	complex*16 y0(0:2),y(0:2,-1:1),x(0:2,-1:1)
	real*8 thp(0:2),thy(0:2)
	integer i,j,ext

	COMMON/IR/LAMBDA,LAMBDA2

	kappa2(a,b,c) = a**2+b**2+c**2-2d0*a*b-2d0*a*c-2d0*b*c

	if (ext.ne.0) then
	  C0_ = C0_ext(q01,q12,q20,m0,m1,m2,ext)
	  return
	endif

        pi   = 4d0*datan(1d0)                                               
	eps  = 1d-17
	ieps = dcmplx(0d0,eps)

	m02 = m0**2
	m12 = m1**2
	m22 = m2**2
	p01 = q01
	p12 = q12
	p20 = q20

C*** Check for IR divergence
c    Permutate until m0=0
10	continue
	if ((m02*m12*m22.eq.0d0).and.(m02.ne.0d0)) then
	  m   = m02
	  m02 = m12
	  m12 = m22
	  m22 = m
	  p   = p01
	  p01 = p12
	  p12 = p20
	  p20 = p
	else
	  goto 20
	endif
	goto 10  
20	continue
	if ((p01.eq.m12).and.(p20.eq.m22).and.(m02.eq.0d0)) goto 500

C****** Regular C0 function
c    Permutate until p01=0
11	continue
	if ((p01*p12*p20.eq.0d0).and.(p01.ne.0d0)) then
	  m   = m02
	  m02 = m12
	  m12 = m22
	  m22 = m
	  p   = p01
	  p01 = p12
	  p12 = p20
	  p20 = p
	else
	  goto 21
	endif
	goto 11  
21	continue
	if (p01.eq.0d0) goto 600
	 
C*** Regular C0 function with p01,p12,p20 =/= 0
	alpha  = sqrt( kappa2(p01,p12,p20) )
	alp(0) = sqrt( kappa2(p12,m12,m22)*(1d0+ieps*sign(1d0,p12)) )
	alp(1) = sqrt( kappa2(p20,m22,m02)*(1d0+ieps*sign(1d0,p20)) )
	alp(2) = sqrt( kappa2(p01,m02,m12)*(1d0+ieps*sign(1d0,p01)) )

	do 99 i=0,2
	  if (alp(i).eq.dcmplx(0d0,0d0)) alp(i) = ieps*abs(alpha)
99	continue

	y0(0)  = ( p12*(p12-p01-p20+2d0*m02-m12-m22)
     &	  - (p20-p01)*(m12-m22)+alpha*(p12-m12+m22) )/2d0/alpha/p12
	y0(1)  = ( p20*(p20-p12-p01+2d0*m12-m22-m02)
     &	  - (p01-p12)*(m22-m02)+alpha*(p20-m22+m02) )/2d0/alpha/p20
	y0(2)  = ( p01*(p01-p20-p12+2d0*m22-m02-m12)
     &	  - (p12-p20)*(m02-m12)+alpha*(p01-m02+m12) )/2d0/alpha/p01

	do 100 j=-1,1,2
	  x(0,j) = (p12-m12+m22+j*alp(0))/2d0/p12
	  x(1,j) = (p20-m22+m02+j*alp(1))/2d0/p20
	  x(2,j) = (p01-m02+m12+j*alp(2))/2d0/p01
	do 100 i=0,2
	  y(i,j) = y0(i)-x(i,j)
100	continue

	do 200 i=0,2
	  thp(i) = 0d0
	  thy(i) = 0d0
	  if (dimag(y(i,+1)*y(i,-1)).le.0d0) thy(i) = 1d0
200	continue
	if (p12.le.0d0) thp(0) = 1d0
	if (p20.le.0d0) thp(1) = 1d0
	if (p01.le.0d0) thp(2) = 1d0

	c0_ = 0d0
	do 300 i=0,2
	do 400 j=-1,1,2
	  c0_ = c0_ + cspen((y0(i)-1d0)/y(i,j)) - cspen(y0(i)/y(i,j))
     &	            + eta(1d0-x(i,j),1d0/y(i,j))*log((y0(i)-1d0)/y(i,j))
     &	            - eta(   -x(i,j),1d0/y(i,j))*log(y0(i)/y(i,j))
400	continue
	  c0_ = c0_ - log((y0(i)-1d0)/y0(i))*(
     &		        eta(-x(i,+1),-x(i,-1))-eta(y(i,+1),y(i,-1))
     &		      - dcmplx(0d0,2d0*pi)*thp(i)*thy(i) )
300	continue
	c0_ = c0_/alpha

	return
600	continue

	if (m02*m12.eq.0d0) goto 700
C*** Regular C0 function with p01=0 and m0,m1=/=0
	alpha   = 1d0+(p12-p20)/(m02-m12-ieps*(m02+m12))
	alp(0)  = sqrt( kappa2(p20,m02,m22)+ieps*p20*(p20-m02-m22) )
	alp(1)  = sqrt( kappa2(p12,m12,m22)+ieps*p12*(p12-m12-m22) )
	x(0,+1) = (p20-m02-m22+alp(0))/2d0/m02
	x(0,-1) = (p20-m02-m22-alp(0))/2d0/m02
	x(1,+1) = (p12-m12-m22+alp(1))/2d0/m12
	x(1,-1) = (p12-m12-m22-alp(1))/2d0/m12
	c0_ = 0d0
	do 610 i=-1,1,2
	do 620 j=0,1
	c0_ = c0_ + (1d0-2d0*j)*( 
     &		cspen(1d0+x(j,i)/alpha) - cspen(1d0+x(j,i)) 
     &		+ eta(1d0/alpha,-x(j,i))*log(1d0+x(j,i)/alpha) )
620	continue
610	continue
	c0_ = c0_ + log(alpha)*log(m02/m12)
	c0_ = c0_/(p20-p12) 

	return
700	continue

C*** Regular C0 function with p01=0 and m0=0
	if (m12.eq.0d0) then
	  m12 = m02
	  m02 = 0d0
	  p   = p12
	  p12 = p20
	  p20 = p
	endif
	alpha   = 1d0+(p12-p20)/(-m12-ieps*m12)
	alp(1)  = sqrt( kappa2(p12,m12,m22)+ieps*p12*(p12-m12-m22) )
	x(0,-1) = m22/(p20-m22+ieps*m12)
	x(1,+1) = (p12-m12-m22+alp(1))/2d0/m12
	x(1,-1) = (p12-m12-m22-alp(1))/2d0/m12
	c0_ = 0d0
	do 710 i=-1,1,2
	c0_ = c0_ - cspen(1d0+x(1,i)/alpha) + cspen(1d0+x(1,i)) 
     &		  - eta(1d0/alpha,-x(1,i))*log(1d0+x(1,i)/alpha) 
710	continue
	c0_ = c0_ + cspen(1d0+x(0,-1)/alpha) - cspen(1d0+x(0,-1)) 
     &		  + eta(1d0/alpha,-x(0,-1))*log(1d0+x(0,-1)/alpha) 
	c0_ = c0_ + log(alpha)*log((m22-ieps*m12-p20)/m12)
     &		  - log(alpha)**2/2d0
	c0_ = c0_/(p20-p12) 

	return
500	continue

C*** IR-divergent C0 function
	sc  = p12+abs(p12)*ieps
	mm1 = sqrt(m12)
	mm4 = sqrt(m22)
	xs = -4d0*mm1*mm4/(sc-(mm1-mm4)**2) /
     &	     ( sqrt(1d0-4d0*mm1*mm4/( sc-(mm1-mm4)**2))+1d0 )**2 
	c0_ = xs/mm1/mm4/(1d0-xs**2)*(
     &	    log(xs)*( -log(xs)/2d0+2d0*log(1d0-xs**2)
     &		    -log(lambda2/mm1/mm4) )
     &	  - pi**2/6d0+cspen(xs**2)+log(mm1/mm4)**2/2d0
     &	  + cspen(1d0-xs*mm1/mm4) + cspen(1d0-xs*mm4/mm1) )

	end

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
        FUNCTION CSPEN(Z)                                              
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C       SPENCE-FUNKTION KOMPLEX, FREI NACH HOLLIK                     C
C---------------------------------------------------------------------C
C       20.07.83    LAST CHANGED 10.05.89        ANSGAR DENNER        C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
        COMPLEX*16 CSPEN,W,SUM,Z,U                                     
        REAL*8 RZ,AZ,A1                                                
        REAL*8 B(9)/                                                   
     1   0.1666666666666666666666666667D0,                             
     2  -0.0333333333333333333333333333D0,                             
     3   0.0238095238095238095238095238D0,                             
     4  -0.0333333333333333333333333333D0,                             
     5   0.0757575757575757575757575758D0,                             
     6  -0.2531135531135531135531135531D0,                             
     7   1.1666666666666666666666666667D0,                             
     8  -7.09215686274509804D0         ,                               
     9  54.97117794486215539D0         /                               
C     BEACHTE:                 B(N)=B2N                                
C     B(1)=1./6.                                                       
C     B(2)=-1./30.                                                     
C     B(3)=1./42.                                                      
C     B(4)=-1./30.                                                     
C     B(5)=5./66.                                                      
C     B(6)=-691./2730.                                                 
C     B(7)=7./6.                                                       
C     B(8)=-3617./510.                                                 
C     B(9)=43867./798.                                                 
C     B(10)=-174611./330.                                              
C     B(11)=854513./138.                                               
C     PI=3.1415926535897932384                                         
C     PI*PI/6.=1.6449..., PI*PI/3=3.28986...                           
C                                                                      
      Z =Z*DCMPLX(1D0)                                                 
      RZ=DREAL(Z)                                                      
      AZ=CDABS(Z)                                                      
      A1=CDABS(1D0-Z)                                                  
C     IF((SNGL(RZ) .EQ. 0.0) .AND. (SNGL(DIMAG(Z)) .EQ. 0.0)) THEN     
C ---> CHANGED  10.5.89                                                
      IF(AZ .LT. 1D-20) THEN                                           
        CSPEN=-CDLOG(1D0-Z)                                            
        RETURN                                                         
      END IF                                                           
c      IF((SNGL(RZ) .EQ. 1.0) .AND. (SNGL(DIMAG(Z)) .EQ. 0.0)) THEN     
c ---> changed 5.7.94
       IF( (ABS(RZ-1D0).LT.1D-18) .AND. (ABS(DIMAG(Z)).LT.1D-18) ) THEN     
        CSPEN=1.64493406684822643D0                                    
        RETURN                                                         
      END IF                                                           
      IF(RZ.GT.5D-1) GOTO 20                                           
      IF(AZ.GT.1D0) GOTO 10                                            
      W=-CDLOG(1D0-Z)                                                  
      SUM=W-0.25D0*W*W                                                 
      U=W                                                              
      IF(CDABS(U).LT.1D-10) GOTO 2                                     
      DO 1 K=1,9                                                       
      U=U*W*W/DFLOAT(2*K*(2*K+1))                                      
      IF(CDABS(U*B(K)/SUM).LT.1D-20) GOTO 2                            
      SUM=SUM+U*B(K)                                                   
 1    CONTINUE                                                         
 2    CSPEN=SUM                                                        
      RETURN                                                           
10    W=-CDLOG(1D0-1D0/Z)                                              
      SUM=W-0.25D0*W*W                                                 
      U=W                                                              
      IF(CDABS(U).LT.1D-10) GOTO 12                                    
                                                                       
      DO 11 K=1,9                                                      
      U=U*W*W/DFLOAT(2*K*(2*K+1))                                      
      IF(CDABS(B(K)*U/SUM).LT.1D-20) GOTO 12                           
      SUM=SUM+U*B(K)                                                   
11    CONTINUE                                                         
12    CSPEN=-SUM-1.64493406684822643D0-.5D0*CDLOG(-Z)**2               
      RETURN                                                           
20    IF(A1.GT.1D0) GOTO 30                                            
      W=-CDLOG(Z)                                                      
      SUM=W-0.25D0*W*W                                                 
      U=W                                                              
      IF(CDABS(U).LT.1D-10) GOTO 22                                    
      DO 21 K=1,9                                                      
      U=U*W*W/DFLOAT(2*K*(2*K+1))                                      
      IF(CDABS(U*B(K)/SUM).LT.1D-20) GOTO 22                           
      SUM=SUM+U*B(K)                                                   
21    CONTINUE                                                         
22    CSPEN=-SUM+1.64493406684822643D0-CDLOG(Z)*CDLOG(1D0-Z)           
      RETURN                                                           
30    W=CDLOG(1D0-1D0/Z)                                               
      SUM=W-0.25D0*W*W                                                 
      U=W                                                              
      IF(CDABS(U).LT.1D-10) GOTO 32                                    
      DO 31 K=1,9                                                      
      U=U*W*W/DFLOAT(2*K*(2*K+1))                                      
      IF(CDABS(U*B(K)/SUM).LT.1D-20) GOTO 32                           
      SUM=SUM+U*B(K)                                                   
31    CONTINUE                                                         
32    CSPEN=SUM+3.28986813369645287D0                                  
     *               +.5D0*CDLOG(Z-1D0)**2-CDLOG(Z)*CDLOG(1D0-Z)       
50    CONTINUE                                                         
        END                                                            

***********************************************************************
        FUNCTION ETA(C1,C2)                                            
***********************************************************************
*       COMPLEX ETA-FUNKTION                                           
*---------------------------------------------------------------------*
*       8.06.90    ANSGAR DENNER                                       
***********************************************************************
        IMPLICIT   LOGICAL(A-Z)                                        
        COMPLEX*16 ETA,C1,C2                                           
        REAL*8     PI,IM1,IM2,IM12                                     
                                                                       
        PI     = 4D0*DATAN(1D0)                                        
        IM1    = DIMAG(C1)                                             
        IM2    = DIMAG(C2)                                             
        IM12   = DIMAG(C1*C2)                                          

	if (((IM1.eq.0d0).and.(DREAL(C1).lt.0d0)).or.
     &	    ((IM2.eq.0d0).and.(DREAL(C2).lt.0d0)).or.
     &	    ((IM12.eq.0d0).and.(DREAL(C1*C2).lt.0d0))) then
	  write(*,*) 'eta function on cut !!!'
	  write(*,*) 'C1    = ',C1
	  write(*,*) 'C2    = ',C2
	  write(*,*) 'C1*C2 = ',C1*C2
	  stop
	endif
                                                                       
        IF(IM1.LT.0D0.AND.IM2.LT.0D0.AND.IM12.GT.0D0) THEN             
            ETA = DCMPLX(0D0,2D0*PI)                                   
        ELSE IF (IM1.GT.0D0.AND.IM2.GT.0D0.AND.IM12.LT.0D0) THEN       
            ETA = DCMPLX(0D0,-2D0*PI)                                  
        ELSE                                                           
            ETA = DCMPLX(0D0)                                          
        END IF                                                         
        END                                                            

***********************************************************************
        FUNCTION ETAS(Y,R,RS)                                            
***********************************************************************
*       MODIFIED ETA-FUNKTION                                           
*---------------------------------------------------------------------*
*       18.1.94   Stefan Dittmaier                                       
***********************************************************************
        IMPLICIT   LOGICAL(A-Z)                                        
        COMPLEX*16 ETA,ETAS,Y,R,RS
        REAL*8     PI,IMY,IMRS
                                                                       
        PI     = 4D0*DATAN(1D0)                                        

	IF( DIMAG(R).NE.0D0 ) THEN
	    ETAS = ETA(Y,R)
	ELSE	    
	    IF( DREAL(R).GT.0D0 ) THEN
		ETAS = DCMPLX(0D0,0D0)
	    ELSE
	 	IMY  = DIMAG(Y)
		IMRS = DIMAG(RS)
		ETAS = 2D0*DCMPLX(0D0,PI)*(
     *			(1D0+SIGN(1D0,-IMY))*(1D0+SIGN(1D0,-IMRS))-
     *			(1D0+SIGN(1D0, IMY))*(1D0+SIGN(1D0, IMRS))
     *					  )/4D0
	    ENDIF
	ENDIF
        END                                                            

***********************************************************************
        FUNCTION SQE(A,B,C)                                            
***********************************************************************
*       SOLUTION OF QUADRATIC EQUATION				      *
*---------------------------------------------------------------------*
*       13.1.92  Stefan Dittmaier				      *
***********************************************************************
        IMPLICIT REAL*8 (A-Z)                                        
        COMPLEX*16 A,B,C,SQE,X1,X2

	X1=(-B+SQRT(B**2-4D0*A*C))/2D0/A
	X2=(-B-SQRT(B**2-4D0*A*C))/2D0/A

	IF (ABS(X1).GT.ABS(X2)) THEN
	   SQE=X1
	ELSE
	   SQE=X2
	ENDIF

        END                                                            

************************************************************************
        FUNCTION D0_(P1,P2,P3,P4,P12,P23,M1,M2,M3,M4,ext) 
************************************************************************
*  SCALAR 4-POINT FUNCTION                                             *
*  P1,P2,P3,P4 = SQUARED EXTERNAL MOMENTA			       *
*  P12 = (p1+p2)**2,  P23 = (p2+p3)**2				       *
*----------------------------------------------------------------------*
*	General result from                                            *
*        A.Denner, U.Nierste and R.Scharf, Nucl. Phys. B367 (1991) 637 *
*	IR-divergent case from                                         *
*        W.Beenakker and A.Denner, Nucl. Phys. B338 (1990) 349         *
*----------------------------------------------------------------------*
*  2.1.92  Stefan Dittmaier	         			       *
************************************************************************
        IMPLICIT REAL*8 (A-Z) 
	REAL*8 M(4),P(4,4),K(4,4)                                     
	COMPLEX*16 A1,A2,A3,A4,SWAP
	COMPLEX*16 SS(4), XX(2), X(2,4),RS(4,4)
	COMPLEX*16 S0(4),XX0(2),X0(2,4), R(4,4),G(2)
        COMPLEX*16 D0_,D0_ext,CSPEN,ETA,SQE,ETAS
	COMPLEX*16 AA,BB,CC,DD,IEPS,H,HH,L1,L2,L3,L4
	COMPLEX*16 SC,TC,XS,X2,X3,q2c,q3c,Y
	INTEGER I,J,ext

	COMMON/IR/LAMBDA,LAMBDA2


	if (ext.ne.0) then
	  D0_ = D0_ext(P1,P2,P3,P4,P12,P23,M1,M2,M3,M4,ext)
	  return
	endif

        PI = 4D0*DATAN(1D0)                                               

        MM1=M1    
        MM2=M2    
        MM3=M3    
        MM4=M4    
        M12=M1*M1 
        M22=M2*M2 
        M32=M3*M3 
        M42=M4*M4 
        Q1=P1 
        Q2=P2   
        Q3=P3   
	Q4=P4
        Q12=P12   
        Q23=P23

C	IS AT LEAST ONE MASS ZERO ???
	IF (MM1*MM2*MM3*MM4.NE.0D0) GOTO 130

C--->	****** IR-divergent CASE ******
	IF ( ((Q1.EQ.M12).AND.(Q2.EQ.M32).AND.(M22.EQ.0D0)).OR.
     *	     ((Q2.EQ.M22).AND.(Q3.EQ.M42).AND.(M32.EQ.0D0)).OR.
     *	     ((Q3.EQ.M32).AND.(Q4.EQ.M12).AND.(M42.EQ.0D0)).OR.
     *	     ((Q4.EQ.M42).AND.(Q1.EQ.M22).AND.(M12.EQ.0D0)) ) goto 50

C--->	****** REGULAR CASE with at least one mass zero ******

C	PERMUTATE UNTIL MM3=0D0
	GOTO 20
10	CONTINUE
	MM0=MM1
	MM1=MM2
	MM2=MM3
	MM3=MM4
	MM4=MM0
	M02=M12
	M12=M22
	M22=M32
	M32=M42
	M42=M02
	Q00=Q12
	Q12=Q23
	Q23=Q00
	Q0=Q1
	Q1=Q2
	Q2=Q3
	Q3=Q4
	Q4=Q0
20	IF (MM3.NE.0D0) GOTO 10
C	ONLY MM3 IS ZERO
	IF (MM1*MM2*MM4.NE.0D0) GOTO 30
C	ONLY MM3 AND MM4 ARE ZERO ==> 3->2, 4->3...   
	IF ((MM1*MM2.NE.0D0).AND.(MM4.EQ.0D0)) GOTO 10
C	ONLY MM2 AND MM3 ARE ZERO
	IF ((MM1*MM4.NE.0D0).AND.(MM2.EQ.0D0)) GOTO 40
C	ONLY MM1 AND MM3 ARE ZERO ==> m1 <-> m2
	IF ((MM2*MM4.NE.0D0).AND.(MM1.EQ.0D0)) then
	  mm0 = mm1
	  mm1 = mm2
	  mm2 = mm0
	  q0  = q2
	  q2  = q12
	  q12 = q0
	  q0  = q4
	  q4  = q23
	  q23 = q0
	  goto 40
	endif
C 	check whether all masses are zero
	if ((mm1.eq.0d0).and.(mm2.eq.0d0).and.(mm4.eq.0d0)) then
	  WRITE(*,*) 'D0 case with all mi=0 not implemented !'
	  stop
	endif
C 	permutate until mm1 is non-zero
	if (mm1.eq.0d0) goto 10
	goto 70

C	****** NO MASS EQUAL TO ZERO ******
130	CONTINUE
	EPS=1D-17
	IEPS=DCMPLX(0D0,EPS)

	IF( ABS((MM1**2+MM3**2-Q12)/MM1/MM3).LT.2D0 ) THEN
C	R13 WOULD BE NOT REAL. -> PERMUTATION! -> R(2,4) IS NOT REAL.
	   M(1)=MM2
	   M(2)=MM3
	   M(3)=MM4
	   M(4)=MM1
	   P(1,2)=Q2
	   P(1,3)=Q23
	   P(1,4)=Q1
	   P(2,3)=Q3
	   P(2,4)=Q12
	   P(3,4)=Q4
	ELSE
C	R(1,3) IS REAL.
	   M(1)=MM1
	   M(2)=MM2
	   M(3)=MM3
	   M(4)=MM4
	   P(1,2)=Q1
	   P(1,3)=Q12
	   P(1,4)=Q4
	   P(2,3)=Q2
	   P(2,4)=Q23
	   P(3,4)=Q3
	ENDIF

	DO 11 J=2,4
	DO 11 I=1,J-1
	K(I,J)=(M(I)**2+M(J)**2-P(I,J))/M(I)/M(J)
	R(I,J) =SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),0D0),
     *	            DCMPLX(1D0,0D0))
	IF( DIMAG(R(I,J)).EQ.0D0 ) THEN
	   RS(I,J)=SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),EPS),
     *	               DCMPLX(1D0,0D0))
	ELSE
	   RS(I,J)=R(I,J)
	ENDIF
11	CONTINUE

	SS(1)=RS(1,2)
	SS(2)=RS(2,3)
	SS(3)=RS(3,4)
	SS(4)=RS(1,4)
	S0(1)=R(1,2)
	S0(2)=R(2,3)
	S0(3)=R(3,4)
	S0(4)=R(1,4)
	AA=K(3,4)/R(2,4)+R(1,3)*K(1,2)-K(1,4)*R(1,3)/R(2,4)-K(2,3)
	BB=(R(2,4)-1D0/R(2,4))*(R(1,3)-1D0/R(1,3))
     *		+K(1,2)*K(3,4)-K(1,4)*K(2,3)
	CC=K(1,2)/R(1,3)+R(2,4)*K(3,4)-K(1,4)*R(2,4)/R(1,3)-K(2,3)
	DD=K(2,3)-R(1,3)*K(1,2)-R(2,4)*K(3,4)+R(1,3)*R(2,4)*K(1,4)
	XX(1)=SQE(AA,BB,CC+IEPS*DD)
	XX(2)=(CC+IEPS*DD)/AA/XX(1)
	XX0(1)=SQE(AA,BB,CC)
	XX0(2)=CC/AA/XX0(1)

c	XX(1)=XX0(1)-IEPS*DD/(XX0(1)-XX0(2))
c	XX(2)=XX0(2)+IEPS*DD/(XX0(1)-XX0(2))

c	IF (ABS(DREAL(XX0(1)-XX(2))).LT.ABS(DREAL(XX0(1)-XX(1)))) THEN
	IF (ABS(XX0(1)-XX(2)).LT.ABS(XX0(1)-XX(1))) THEN
	  SWAP  =XX0(1)
	  XX0(1)=XX0(2)
	  XX0(2)=SWAP
	ENDIF

	DO 12 I=1,2
	G(I)  =SIGN( 1D0,DREAL(AA*(XX(I)-XX(3-I))) )
	 X(I,1)= XX(I)/R(2,4)
	X0(I,1)=XX0(I)/R(2,4)
	 X(I,2)= XX(I)/R(2,4)*R(1,3)
	X0(I,2)=XX0(I)/R(2,4)*R(1,3)
	 X(I,3)= XX(I)*R(1,3)
	X0(I,3)=XX0(I)*R(1,3)
	 X(I,4)= XX(I)
	X0(I,4)=XX0(I)
12	CONTINUE

	D0_ = DCMPLX(0D0,0D0)
	DO 13 I=1,2
	DO 13 J=1,4
	A1 = 1D0+X0(I,J)*S0(J) + ABS(1D0+X0(I,J)*S0(J))*IEPS*
     *				  SIGN(1D0,DIMAG(X(I,J)*SS(J)))
	A2 = 1D0+X0(I,J)/S0(J) + ABS(1D0+X0(I,J)/S0(J))*IEPS*
     *				  SIGN(1D0,DIMAG(X(I,J)/SS(J)))
	D0_ = D0_ + (-1D0)**(I+J)*(
     *		CSPEN(A1)+ETA(-X(I,J),SS(J))*LOG(A1)
     *	       +CSPEN(A2)+ETA(-X(I,J),1D0/SS(J))*LOG(A2)     )
13	CONTINUE

	IF( DIMAG(R(1,3)).EQ.0D0 ) THEN
	DO 14 I=1,2
	   A1 = (K(1,3)-2D0*R(1,3))/XX0(I)
     *		      -R(1,3)*K(1,4)+K(3,4)
     	   A2 = ((K(2,4)-2D0*R(2,4))*R(1,3)*XX0(I)
     *		      -R(2,4)*K(3,4)+K(2,3))/DD
	   A3 = (K(1,3)-2D0*R(1,3))*R(2,4)/XX0(I)
     *		      -R(1,3)*K(1,2)+K(2,3)
	   A4 = ((K(2,4)-2D0*R(2,4))*XX0(I)
     *		      -R(2,4)*K(1,4)+K(1,2))/DD
	   L1 = LOG( A1-ABS(A1)*IEPS )
     	   L2 = LOG( A2+ABS(A2)*IEPS*G(I)*SIGN(1D0,DREAL(R(1,3))
     *				        	  *DIMAG(RS(2,4))) ) 
	   L3 = LOG( A3-ABS(A3)*IEPS )
	   L4 = LOG( A4+ABS(A4)*IEPS*G(I)*SIGN(1D0,DIMAG(RS(2,4))) ) 

	   D0_ = D0_ + (3D0-2D0*I)*(
     *		 ETAS(-XX(I),R(1,3),RS(1,3))
     *		   *( LOG(R(1,3)*XX(I)) + L1 + L2 )
     *		+ETAS(-XX(I),1D0/R(2,4),1D0/RS(2,4))
     *		   *( LOG(XX(I)/R(2,4)) + L3 + L4 )
     *		-( ETAS(-XX(I),R(1,3)/R(2,4),RS(1,3)/RS(2,4))
     *		  +ETA(RS(1,3),1D0/RS(2,4)) )
     *		   *( LOG(XX(I)*R(1,3)/R(2,4)) + L3 + L2 )
     *	  	+ETA(RS(1,3),1D0/RS(2,4))
     *		   *ETAS(-XX(I),-R(1,3)/R(2,4),-RS(1,3)/RS(2,4))   )
14	CONTINUE
	ELSE
	DO 15 I=1,2
	   L1 = LOG( R(2,4)/XX0(I)+XX0(I)/R(2,4)+K(1,2)
     *		     -XX0(I)/R(2,4)*EPS*BB*G(I) )
	   L2 = LOG( R(1,3)*XX0(I)+1D0/XX0(I)/R(1,3)+K(3,4)
     *		     -XX0(I)*R(1,3)*EPS*BB*G(I) )
	   L3 = LOG( R(1,3)/R(2,4)*XX0(I)+R(2,4)/XX0(I)/R(1,3)+K(2,3)
     *		     -XX0(I)*R(1,3)/R(2,4)*EPS*BB*G(I) )
	   D0_ = D0_ + (3D0-2D0*I)*(
     *		+ETA(-XX(I),1D0/R(2,4))
     *		   *( LOG(XX(I)/R(2,4)) + L1 )
     *		+ETA(-XX(I),R(1,3))
     *		   *( LOG(R(1,3)*XX(I)) + L2 )
     *		-( ETA(-XX(I),R(1,3)/R(2,4))
     *		  +ETA(R(1,3),1D0/R(2,4)) )
     *		   *( LOG(XX(I)*R(1,3)/R(2,4)) + L3 )
     *	  	+ETA(R(1,3),1D0/R(2,4))
     *		   *ETA(-XX(I),-R(1,3)/R(2,4))
     *		   *(1D0-G(I)*SIGN(1D0,DREAL(BB)))	    )
15	CONTINUE
	ENDIF

	D0_ = D0_/M(1)/M(2)/M(3)/M(4)/AA/(XX(1)-XX(2))
	RETURN

C	****** ONLY MM3 IS ZERO ******
30	CONTINUE
	EPS=1D-17
	IEPS=DCMPLX(0D0,EPS)
	M(1)=MM1
	M(2)=MM2
	M(3)=10D0
	M(4)=MM4
	P(1,2)=Q1
	P(1,3)=Q12
	P(1,4)=Q4
	P(2,3)=Q2
	P(2,4)=Q23
	P(3,4)=Q3
	DO 1 J=2,4
	DO 1 I=1,J-1
	K(I,J)=(M(I)**2+M(J)**2-P(I,J))/M(I)/M(J)
	IF (I.EQ.3) K(I,J)=K(I,J)-M(I)/M(J)
	IF (J.EQ.3) K(I,J)=K(I,J)-M(J)/M(I)
	R(I,J) =SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),0D0),
     *	            DCMPLX(1D0,0D0))
	IF( DIMAG(R(I,J)).EQ.0D0 ) THEN
	   RS(I,J)=SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),EPS),
     *	               DCMPLX(1D0,0D0))
	ELSE
	   RS(I,J)=R(I,J)
	ENDIF
1	CONTINUE
	SS(1)=RS(1,2)
	SS(2)=RS(2,3)
	SS(3)=RS(3,4)
	SS(4)=RS(1,4)
	AA=K(3,4)/R(2,4)-K(2,3)
	BB=K(1,3)*(1D0/R(2,4)-R(2,4))+K(1,2)*K(3,4)-K(1,4)*K(2,3)
	CC=K(1,2)*K(1,3)-K(1,3)*K(1,4)*R(2,4)+R(2,4)*K(3,4)-K(2,3)
	DD=K(2,3)-R(2,4)*K(3,4)
	XX(1)=SQE(AA,BB,CC+IEPS*DD)
	XX(2)=(CC+IEPS*DD)/AA/XX(1)
	DO 2 I=1,2
	X(I,1)=XX(I)/R(2,4)
	X(I,2)=XX(I)/R(2,4)*R(1,3)
	X(I,3)=XX(I)*R(1,3)
	X(I,4)=XX(I)
2	CONTINUE
	D0_ = DCMPLX(0D0,0D0)
	DO 3 I=1,2
	D0_ = D0_ + (2D0*I-3D0)*(
     *		CSPEN(1D0+SS(4)*X(I,4))
     *	       -CSPEN(1D0+SS(1)*X(I,1))
     *	       +CSPEN(1D0+X(I,4)/SS(4))
     *	       -CSPEN(1D0+X(I,1)/SS(1))
     *	       +ETA(-X(I,4),SS(4))*LOG(1D0+SS(4)*X(I,4))
     *	       -ETA(-X(I,1),SS(1))*LOG(1D0+SS(1)*X(I,1))
     *	       +ETA(-X(I,4),1D0/SS(4))*LOG(1D0+X(I,4)/SS(4))
     *	       -ETA(-X(I,1),1D0/SS(1))*LOG(1D0+X(I,1)/SS(1))
     *	       -CSPEN(1D0+X(I,4)*(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	       +CSPEN(1D0+X(I,1)*(K(2,3)-IEPS)/(K(1,3)-IEPS))
     *	       -ETA(-X(I,4),(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	           *LOG(1D0+X(I,4)*(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	       +ETA(-X(I,1),(K(2,3)-IEPS)/(K(1,3)-IEPS))
     *	           *LOG(1D0+X(I,1)*(K(2,3)-IEPS)/(K(1,3)-IEPS))   )
	IF (DIMAG(R(2,4)).NE.0D0) THEN
	   H=ETA(-1D0/XX(I),R(2,4))
	ELSE
	   H=DCMPLX(0D0,0D0)
	   IF (DREAL(R(2,4)).LT.0D0) THEN
	      HH=-1D0/XX(I)
	      IM1=DIMAG(HH)
	      IM2=DIMAG(RS(2,4))
	      IF ((IM1.GT.0D0).AND.(IM2.GT.0D0)) THEN
	         H=-DCMPLX(0D0,2D0*PI)
	      ENDIF
	      IF ((IM1.LT.0D0).AND.(IM2.LT.0D0)) THEN
	         H=+DCMPLX(0D0,2D0*PI)
	      ENDIF
	   ENDIF
	ENDIF
	D0_ = D0_ + (2D0*I-3D0)*
     *	          H*( LOG( (K(1,2)-R(2,4)*K(1,4)
     *			  +XX(I)*(1D0/R(2,4)-R(2,4)))/DD )
     *		     +LOG(K(1,3)-IEPS) )
3	CONTINUE
	D0_ = D0_/M(1)/M(2)/M(3)/M(4)/AA/(XX(1)-XX(2))
	RETURN

C	****** ONLY MM2 AND MM3 ARE ZERO ******
40	CONTINUE
	EPS=1D-17
	IEPS=DCMPLX(0D0,EPS)

	M(1)=MM1
	M(2)=10D0
	M(3)=10D0
	M(4)=MM4
	P(1,2)=Q1
	P(1,3)=Q12
	P(1,4)=Q4
	P(2,3)=Q2
	P(2,4)=Q23
	P(3,4)=Q3
	DO 4 J=2,4
	DO 4 I=1,J-1
	K(I,J)=(M(I)**2+M(J)**2-P(I,J))/M(I)/M(J)
	IF (I.EQ.2) K(I,J)=K(I,J)-M(I)/M(J)
	IF (J.EQ.2) K(I,J)=K(I,J)-M(J)/M(I)
	IF (I.EQ.3) K(I,J)=K(I,J)-M(I)/M(J)
	IF (J.EQ.3) K(I,J)=K(I,J)-M(J)/M(I)
	R(I,J) =SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),0D0),
     *	            DCMPLX(1D0,0D0))
	IF( DIMAG(R(I,J)).EQ.0D0 ) THEN
	   RS(I,J)=SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),EPS),
     *	               DCMPLX(1D0,0D0))
	ELSE
	   RS(I,J)=R(I,J)
	ENDIF
4	CONTINUE
	SS(1)=RS(1,2)
	SS(2)=RS(2,3)
	SS(3)=RS(3,4)
	SS(4)=RS(1,4)
	AA=K(2,4)*K(3,4)-K(2,3)
	BB=K(1,3)*K(2,4)+K(1,2)*K(3,4)-K(1,4)*K(2,3)
	CC=K(1,2)*K(1,3)-K(2,3)
	DD=K(2,3)
	XX(1)=SQE(AA,BB,CC+IEPS*DD)
	XX(2)=(CC+IEPS*DD)/AA/XX(1)
	DO 5 I=1,2
	X(I,1)=XX(I)/R(2,4)
	X(I,2)=XX(I)/R(2,4)*R(1,3)
	X(I,3)=XX(I)*R(1,3)
	X(I,4)=XX(I)
5	CONTINUE
	D0_ = DCMPLX(0D0,0D0)
	DO 6 I=1,2
	D0_ = D0_ + (2D0*I-3D0)*(
     *		CSPEN(1D0+SS(4)*X(I,4))
     *	       +CSPEN(1D0+X(I,4)/SS(4))
     *	       +ETA(-X(I,4),SS(4))*LOG(1D0+SS(4)*X(I,4))
     *	       +ETA(-X(I,4),1D0/SS(4))*LOG(1D0+X(I,4)/SS(4))
     *	       -CSPEN(1D0+XX(I)*(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	       -CSPEN(1D0+XX(I)*(K(2,4)-IEPS)/(K(1,2)-IEPS))
     *	       -ETA(-XX(I),(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	           *LOG(1D0+XX(I)*(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	       -ETA(-XX(I),(K(2,4)-IEPS)/(K(1,2)-IEPS))
     *	           *LOG(1D0+XX(I)*(K(2,4)-IEPS)/(K(1,2)-IEPS)) 
     *	       +LOG(-XX(I))*( LOG(K(1,2)-IEPS)
     *			     +LOG(K(1,3)-IEPS)-LOG(K(2,3)-IEPS) ) )
6	CONTINUE
	D0_ = D0_/M(1)/M(2)/M(3)/M(4)/AA/(XX(1)-XX(2))

	return

C	****** ONLY MM1 IS NON-ZERO ******
70	CONTINUE
	EPS=1D-17
	IEPS=DCMPLX(0D0,EPS)

	M(1)=MM1
	M(2)=10D0
	M(3)=10D0
	M(4)=10D0
	P(1,2)=Q1
	P(1,3)=Q12
	P(1,4)=Q4
	P(2,3)=Q2
	P(2,4)=Q23
	P(3,4)=Q3
	k(1,2) = (m(1)**2-p(1,2))/m(1)/m(2)
	k(1,3) = (m(1)**2-p(1,3))/m(1)/m(3)
	k(1,4) = (m(1)**2-p(1,4))/m(1)/m(4)
	k(2,3) = -p(2,3)/m(2)/m(3)
	k(2,4) = -p(2,4)/m(2)/m(4)
	k(3,4) = -p(3,4)/m(3)/m(4)
	DO 74 J=2,4
	DO 74 I=1,J-1
	R(I,J) =SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),0D0),
     *	            DCMPLX(1D0,0D0))
	IF( DIMAG(R(I,J)).EQ.0D0 ) THEN
	   RS(I,J)=SQE(DCMPLX(1D0,0D0),DCMPLX(-K(I,J),EPS),
     *	               DCMPLX(1D0,0D0))
	ELSE
	   RS(I,J)=R(I,J)
	ENDIF
74	CONTINUE
	AA=K(2,4)*K(3,4)
	BB=K(1,3)*K(2,4)+K(1,2)*K(3,4)-K(1,4)*K(2,3)
	CC=K(1,2)*K(1,3)-K(2,3)
	DD=K(2,3)
	XX(1)=SQE(AA,BB,CC+IEPS*DD)
	XX(2)=(CC+IEPS*DD)/AA/XX(1)
	D0_ = DCMPLX(0D0,0D0)
	DO 76 I=1,2
	D0_ = D0_ + (2D0*I-3D0)*(
     *	        CSPEN(1D0+(K(1,4)-IEPS)*xx(i))
     *	       +eta(-xx(i),K(1,4)-IEPS)*log(1D0+(K(1,4)-IEPS)*xx(i))
     *	       -CSPEN(1D0+XX(I)*(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	       -CSPEN(1D0+XX(I)*(K(2,4)-IEPS)/(K(1,2)-IEPS))
     *	       -ETA(-XX(I),(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	           *LOG(1D0+XX(I)*(K(3,4)-IEPS)/(K(1,3)-IEPS))
     *	       -ETA(-XX(I),(K(2,4)-IEPS)/(K(1,2)-IEPS))
     *	           *LOG(1D0+XX(I)*(K(2,4)-IEPS)/(K(1,2)-IEPS)) 
     *	       +LOG(-XX(I))*( LOG(K(1,2)-IEPS)
     *			     +LOG(K(1,3)-IEPS)-LOG(K(2,3)-IEPS) ) )
76	CONTINUE
	D0_ = D0_/M(1)/M(2)/M(3)/M(4)/AA/(XX(1)-XX(2))
	return

C	****** general IR-divergent D0-function ******
C	16.4.97 Stefan Dittmaier
50	CONTINUE
C	PERMUTATE UNTIL MM1 IS THE PHOTON
	GOTO 52
51	CONTINUE
	MM0=MM1
	MM1=MM2
	MM2=MM3
	MM3=MM4
	MM4=MM0
	M02=M12
	M12=M22
	M22=M32
	M32=M42
	M42=M02
	Q00=Q12
	Q12=Q23
	Q23=Q00
	Q0=Q1
	Q1=Q2
	Q2=Q3
	Q3=Q4
	Q4=Q0
52	IF ((mm1.ne.0d0).or.(q1.ne.m22).or.(q4.ne.m42)) GOTO 51

	EPS=1D-17
	IEPS=DCMPLX(0D0,EPS)
	sc  = q23+abs(q23)*ieps
	tc  = q12+abs(q12)*ieps
	q2c = q2+abs(q2)*ieps
	q3c = q3+abs(q3)*ieps
	xs = -4d0*mm2*mm4/(sc-(mm2-mm4)**2) /
     &	     ( sqrt(1d0-4d0*mm2*mm4/( sc-(mm2-mm4)**2))+1d0 )**2 

	if (mm3.eq.0d0) goto 60

C	*** general case ***
	if (q2.ne.(mm2-mm3)**2) then
	  x2 = -4d0*mm2*mm3/(q2c-(mm2-mm3)**2) /
     &	     ( sqrt(1d0-4d0*mm2*mm3/( q2c-(mm2-mm3)**2))+1d0 )**2 
	else
	  x2 = 1d0
	endif
	if (q3.ne.(mm4-mm3)**2) then
	  x3 = -4d0*mm4*mm3/(q3c-(mm4-mm3)**2) /
     &	     ( sqrt(1d0-4d0*mm4*mm3/( q3c-(mm4-mm3)**2))+1d0 )**2 
	else
	  x3 = 1d0
	endif

	d0_ = xs/mm2/mm4/(q12-m32)/(1d0-xs**2)*(
     &	 2d0*log(xs)*(log(1d0-xs**2)-log(lambda*mm3/(m32-tc)))
     &	+pi**2/2d0+cspen(xs**2)+log(x2)**2+log(x3)**2
     &	-cspen(xs*x2*x3)-(log(xs)+log(x2)+log(x3))*log(1d0-xs*x2*x3)
     &	-cspen(xs*x2/x3)-(log(xs)+log(x2)-log(x3))*log(1d0-xs*x2/x3)
     &	-cspen(xs/x2*x3)-(log(xs)-log(x2)+log(x3))*log(1d0-xs/x2*x3)
     &	-cspen(xs/x2/x3)-(log(xs)-log(x2)-log(x3))*log(1d0-xs/x2/x3) )
	return

60	continue
C	*** special case: mass mm3 opposite to photon is 0 ***
	if ((q2.eq.m22).or.(q3.eq.m42)) goto 61
	Y = mm2/mm4*(q3c-m42)/(q2c-m22)
	d0_ = xs/mm2/mm4/q12/(1d0-xs**2)*(
     &	 log(xs)*( -log(xs)/2d0+2d0*log(1d0-xs**2)-log(lambda2/mm2/mm4)
     &		   -log((q2-m22)/tc)-log((q3-m42)/tc) )
     &	+pi**2/6d0+cspen(xs**2)+log(y)**2/2d0
     &	-cspen(xs*y)-(log(xs)+log(y))*log(1d0-xs*y)
     &	-cspen(xs/y)-(log(xs)-log(y))*log(1d0-xs/y) )
	return

61	continue
C	*** special case: doubly IR-divergent D0 ***
	if ((q2.eq.m22).and.(q3.eq.m42)) then
	d0_ = -xs/mm2/mm4/q12/(1d0-xs**2)*2d0*log(xs)*log(-lambda2/tc)
	else
	  write(*,*) 'Special case of IR-divergent D0 not implemented!'
	endif

	END

***********************************************************************
	SUBROUTINE B012_(P2,M0,M1,B0,B1,B2,rank,ext)
***********************************************************************
*	2-POINT TENSOR COEFFICIENTS
*	COMPLEX VALUED
*----------------------------------------------------------------------
*	26.6.98 Stefan Dittmaier
***********************************************************************
        IMPLICIT REAL*8 (A-Z)                                         
        COMPLEX*16 B0,B1(1),B2(0:2,0:2)
        COMPLEX*16 B0_,B1_,B20_,B21_
	integer rank,ext

	B0      =  B0_(P2,M0,M1,ext)
	if (rank.eq.0) return
	B1(1)   =  B1_(P2,M0,M1,ext)
	if (rank.eq.1) return
	B2(0,0) = B20_(P2,M0,M1,ext)
	B2(1,1) = B21_(P2,M0,M1,ext)

	end

***********************************************************************
	SUBROUTINE C0123_(P1,P2,P3,M1,M2,M3,
     &			  C0,C1,C2,C3,B0,B1,B2,rank,ext)
***********************************************************************
*	3-POINT TENSOR COEFFICIENTS
*	COMPLEX VALUED
*----------------------------------------------------------------------
*	28.1.94 Stefan Dittmaier
***********************************************************************
        IMPLICIT REAL*8 (A-Z)                                         
        COMPLEX*16   B0,  B1(1),  B2(0:2,0:2)
        COMPLEX*16 B0_2,B1_2(1),B2_2(0:2,0:2)
        COMPLEX*16 B0_3,B1_3(1),B2_3(0:2,0:2)
        COMPLEX*16 BB,R1(2),R2(2,2),R3X0(2),R3(2,2)
        COMPLEX*16 C0_,C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
	REAL*8 XI(3,3)
	INTEGER I,J,K,rank,ext

	C0 = C0_(P1,P2,P3,M1,M2,M3,ext)

	if (rank.eq.0) return

	Q1  = P1
	Q2  = P3
	Q12 = (P1+P3-P2)/2D0
	DX  = Q1*Q2-Q12**2
	XI(1,1) = Q2/DX
	XI(1,2) = -Q12/DX
	XI(2,1) = XI(1,2)
	XI(2,2) = Q1/DX
	F1 = Q1-M2**2+M1**2
	F2 = Q2-M3**2+M1**2

	CALL B012_(P2,M2,M3,B0  ,B1  ,B2  ,rank-1,ext)
	CALL B012_(P3,M1,M3,B0_2,B1_2,B2_2,rank-1,ext)
	CALL B012_(P1,M1,M2,B0_3,B1_3,B2_3,rank-1,ext)

	R1(1) = (B0_2-B0-F1*C0)/2D0
	R1(2) = (B0_3-B0-F2*C0)/2D0

	DO 1 I=1,2
	C1(I) = 0D0
	DO 1 K=1,2
	C1(I) = C1(I) + XI(I,K)*R1(K)
1	CONTINUE

	if (rank.eq.1) return

	C2(0,0) = M1**2*C0/2D0+(B0+F1*C1(1)+F2*C1(2)+1D0)/4D0

	R2(1,1) = (        B1(1)+B0-F1*C1(1))/2D0 - C2(0,0)
	R2(1,2) = (B1_2(1)-B1(1)   -F1*C1(2))/2D0
	R2(2,1) = (B1_3(1)+B1(1)+B0-F2*C1(1))/2D0
	R2(2,2) = (       -B1(1)   -F2*C1(2))/2D0 - C2(0,0)

	DO 2 I=1,2
	DO 2 J=1,2
	C2(I,J) = 0D0
	DO 2 K=1,2
	C2(I,J) = C2(I,J) + XI(I,K)*R2(K,J)
2	CONTINUE

	if (rank.eq.2) return

	R3X0(1) = (B2_2(0,0)-B2(0,0)-F1*C2(0,0))/2D0
	R3X0(2) = (B2_3(0,0)-B2(0,0)-F2*C2(0,0))/2D0

	DO 30 I=1,2
	C3(0,0,I) = 0D0
	DO 30 K=1,2
	C3(0,0,I) = C3(0,0,I) + XI(I,K)*R3X0(K)
30	CONTINUE

	BB = B2(1,1)+2D0*B1(1)+B0
	R3(1,1) = (         -BB     -F1*C2(1,1))/2D0 
     *					- 2D0*C3(0,0,1)
	R3(1,2) = (B2_2(1,1)-B2(1,1)-F1*C2(2,2))/2D0
	R3(2,1) = (B2_3(1,1)-BB     -F2*C2(1,1))/2D0
	R3(2,2) = (         -B2(1,1)-F2*C2(2,2))/2D0 
     *					- 2D0*C3(0,0,2)

	DO 3 I=1,2
	DO 3 J=1,2
	C3(I,J,J) = 0D0
	DO 3 K=1,2
	C3(I,J,J) = C3(I,J,J) + XI(I,K)*R3(K,J)
3	CONTINUE

	C3(1,1,2) = C3(2,1,1)
c*** remaining non-ordered coefficients
	C3(1,2,1) = C3(2,1,1)
	C3(2,1,2) = C3(1,2,2)
	C3(2,2,1) = C3(1,2,2)

	END

***********************************************************************
	SUBROUTINE D01234_(P1,P2,P3,P4,P23,P12,M1,M2,M3,M4,
     *			   D0,D1,D2,D3,D4,C0,C1,C2,C3,rank,ext)
***********************************************************************
*	4-POINT TENSOR COEFFICIENTS
*	COMPLEX VALUED
* 	Note order of arguments 5 and 6 in D01234_ and D0_ !
*----------------------------------------------------------------------
*	28.1.94 Stefan Dittmaier
***********************************************************************
        IMPLICIT REAL*8 (A-Z)                                         
        COMPLEX*16   B0,  B1(1),  B2(0:2,0:2)
        COMPLEX*16   C0,  C1(2),  C2(0:2,0:2),  C3(0:2,0:2,2)
        COMPLEX*16 C0_2,C1_2(2),C2_2(0:2,0:2),C3_2(0:2,0:2,2)
        COMPLEX*16 C0_3,C1_3(2),C2_3(0:2,0:2),C3_3(0:2,0:2,2)
        COMPLEX*16 C0_4,C1_4(2),C2_4(0:2,0:2),C3_4(0:2,0:2,2)
        COMPLEX*16 CC,S1(3),S2(3,3),S3X0(3),S3(3,3),S3X23(3)
	COMPLEX*16 S4X0(3,3),S4(3,3),S4X233(3),S4X113(3),S4X122(3)
        COMPLEX*16 D0_,D0,D1(3),D2(0:3,0:3),D3(0:3,0:3,3)
	COMPLEX*16 D4(0:3,0:3,0:3,0:3)
	REAL*8 YI(3,3)
	INTEGER I,J,K,rank,ext

	D0 = D0_(P1,P2,P3,P4,P12,P23,M1,M2,M3,M4,ext)

	if (rank.eq.0) return

	Q1  = P1
	Q2  = P12
	Q3  = P4
	Q12 = (P1+P12-P2)/2D0
	Q23 = (P4+P12-P3)/2D0
	Q13 = (P1+P4-P23)/2D0

	DY  = Q1*Q2*Q3+2D0*Q12*Q23*Q13
     *		-Q1*Q23**2-Q2*Q13**2-Q3*Q12**2

	YI(1,1) = (Q2*Q3-Q23**2)/DY
	YI(1,2) = (Q13*Q23-Q3*Q12)/DY
	YI(1,3) = (Q12*Q23-Q2*Q13)/DY
	YI(2,1) = YI(1,2)
	YI(2,2) = (Q1*Q3-Q13**2)/DY
	YI(2,3) = (Q12*Q13-Q1*Q23)/DY
	YI(3,1) = YI(1,3)
	YI(3,2) = YI(2,3)
	YI(3,3) = (Q1*Q2-Q12**2)/DY

	F1 = Q1-M2**2+M1**2
	F2 = Q2-M3**2+M1**2
	F3 = Q3-M4**2+M1**2
             
	CALL C0123_(P2,P3,P23,M2,M3,M4,
     &			C0  ,C1  ,C2  ,C3  ,B0,B1,B2,rank-1,ext)
	CALL C0123_(P12,P3,P4,M1,M3,M4,
     &			C0_2,C1_2,C2_2,C3_2,B0,B1,B2,rank-1,ext)
	CALL C0123_(P1,P23,P4,M1,M2,M4,
     &			C0_3,C1_3,C2_3,C3_3,B0,B1,B2,rank-1,ext)
	CALL C0123_(P1,P2,P12,M1,M2,M3,
     &			C0_4,C1_4,C2_4,C3_4,B0,B1,B2,rank-1,ext)

	S1(1) = (C0_2-C0-F1*D0)/2D0
	S1(2) = (C0_3-C0-F2*D0)/2D0
	S1(3) = (C0_4-C0-F3*D0)/2D0

	DO 1 I=1,3
	D1(I) = 0D0
	DO 1 K=1,3
	D1(I) = D1(I) + YI(I,K)*S1(K)
1	CONTINUE

	if (rank.eq.1) return

	D2(0,0) = M1**2*D0+(C0
     *			+F1*D1(1)+F2*D1(2)+F3*D1(3))/2D0

	CC = C1(1)+C1(2)+C0
	S2(1,1) = (        CC    - F1*D1(1))/2D0 - D2(0,0)
	S2(1,2) = (C1_2(1)-C1(1) - F1*D1(2))/2D0
	S2(1,3) = (C1_2(2)-C1(2) - F1*D1(3))/2D0
	S2(2,1) = (C1_3(1)+CC    - F2*D1(1))/2D0
	S2(2,2) = (       -C1(1) - F2*D1(2))/2D0 - D2(0,0)
	S2(2,3) = (C1_3(2)-C1(2) - F2*D1(3))/2D0
	S2(3,1) = (C1_4(1)+CC    - F3*D1(1))/2D0
	S2(3,2) = (C1_4(2)-C1(1) - F3*D1(2))/2D0
	S2(3,3) = (       -C1(2) - F3*D1(3))/2D0 - D2(0,0)

	DO 2 I=1,3
	DO 2 J=1,3
	D2(I,J) = 0D0
	DO 2 K=1,3
	D2(I,J) = D2(I,J) + YI(I,K)*S2(K,J)
2	CONTINUE

	if (rank.eq.2) return

	S3X0(1) = (C2_2(0,0)-C2(0,0)-F1*D2(0,0))/2D0
	S3X0(2) = (C2_3(0,0)-C2(0,0)-F2*D2(0,0))/2D0
	S3X0(3) = (C2_4(0,0)-C2(0,0)-F3*D2(0,0))/2D0

	DO 30 I=1,3
	D3(0,0,I) = 0D0
	DO 30 K=1,3
	D3(0,0,I) = D3(0,0,I) + YI(I,K)*S3X0(K)
30	CONTINUE

	CC = C2(1,1)+2D0*C2(1,2)+C2(2,2)
     *		+2D0*C1(1)+2D0*C1(2)+C0
	S3(1,1) = (         -CC      - F1*D2(1,1))/2D0 
     *			- 2D0*D3(0,0,1)
	S3(1,2) = (C2_2(1,1)-C2(1,1) - F1*D2(2,2))/2D0
	S3(1,3) = (C2_2(2,2)-C2(2,2) - F1*D2(3,3))/2D0
	S3(2,1) = (C2_3(1,1)-CC      - F2*D2(1,1))/2D0
	S3(2,2) = (         -C2(1,1) - F2*D2(2,2))/2D0
     *			- 2D0*D3(0,0,2)
	S3(2,3) = (C2_3(2,2)-C2(2,2) - F2*D2(3,3))/2D0
	S3(3,1) = (C2_4(1,1)-CC      - F3*D2(1,1))/2D0
	S3(3,2) = (C2_4(2,2)-C2(1,1) - F3*D2(2,2))/2D0
	S3(3,3) = (         -C2(2,2) - F3*D2(3,3))/2D0
     *			- 2D0*D3(0,0,3)

	DO 3 I=1,3
	DO 3 J=1,3
	D3(I,J,J) = 0D0
	DO 3 K=1,3
	D3(I,J,J) = D3(I,J,J) + YI(I,K)*S3(K,J)
3	CONTINUE

	S3X23(1) = (C2_2(1,2)-C2(1,2)-F1*D2(2,3))/2D0
	S3X23(2) = (         -C2(1,2)-F2*D2(2,3))/2D0
     *			- D3(0,0,3)
	S3X23(3) = (         -C2(1,2)-F3*D2(2,3))/2D0
     *			- D3(0,0,2)

	DO 31 I=1,3
	D3(I,2,3) = 0D0
	DO 31 K=1,3
	D3(I,2,3) = D3(I,2,3) + YI(I,K)*S3X23(K)
31	CONTINUE

	D3(1,1,2) = D3(2,1,1)
	D3(1,1,3) = D3(3,1,1)

c*** remaining non-ordered coefficients
	D3(1,3,2) = D3(1,2,3) 
	D3(2,1,3) = D3(1,2,3) 
	D3(2,3,1) = D3(1,2,3) 
	D3(3,1,2) = D3(1,2,3) 
	D3(3,2,1) = D3(1,2,3) 
	D3(2,2,1) = D3(1,2,2)
	D3(3,3,1) = D3(1,3,3)
	D3(3,3,2) = D3(2,3,3)
	D3(1,2,1) = D3(2,1,1)
	D3(1,3,1) = D3(3,1,1)
	D3(2,1,2) = D3(1,2,2)
	D3(2,3,2) = D3(3,2,2)
	D3(3,1,3) = D3(1,3,3)

	if (rank.eq.3) return

	D4(0,0,0,0) = M1**2*D2(0,0)/3D0 + (C2(0,0)
     *			+F1*D3(0,0,1)+F2*D3(0,0,2)
     *			+F3*D3(0,0,3)+1D0/6D0)/6D0

	CC = C3(0,0,1)+C3(0,0,2)+C2(0,0)
	S4X0(1,1) = (            CC
     *			- F1*D3(0,0,1))/2D0 - D4(0,0,0,0)
	S4X0(1,2) = (C3_2(0,0,1)-C3(0,0,1)
     *			- F1*D3(0,0,2))/2D0
	S4X0(1,3) = (C3_2(0,0,2)-C3(0,0,2)
     *			- F1*D3(0,0,3))/2D0
	S4X0(2,1) = (C3_3(0,0,1)+CC
     *			- F2*D3(0,0,1))/2D0
	S4X0(2,2) = (           -C3(0,0,1)
     *			- F2*D3(0,0,2))/2D0 - D4(0,0,0,0)
	S4X0(2,3) = (C3_3(0,0,2)-C3(0,0,2)
     *			- F2*D3(0,0,3))/2D0
	S4X0(3,1) = (C3_4(0,0,1)+CC
     *			- F3*D3(0,0,1))/2D0
	S4X0(3,2) = (C3_4(0,0,2)-C3(0,0,1)
     *			- F3*D3(0,0,2))/2D0
	S4X0(3,3) = (           -C3(0,0,2)
     *			- F3*D3(0,0,3))/2D0 - D4(0,0,0,0)

	DO 40 I=1,3
	DO 40 J=1,3
	D4(0,0,I,J) = 0D0
	DO 40 K=1,3
	D4(0,0,I,J) = D4(0,0,I,J) + YI(I,K)*S4X0(K,J)
40	CONTINUE

	CC = C3(1,1,1)+3D0*C3(1,1,2)+3D0*C3(1,2,2)+C3(2,2,2)
     *		+3D0*C2(1,1)+6D0*C2(1,2)+3D0*C2(2,2)
     *		+3D0*C1(1)+3D0*C1(2)+C0

	S4(1,1) = (            CC        - F1*D3(1,1,1))/2D0
     *			-3D0*D4(0,0,1,1)
	S4(1,2) = (C3_2(1,1,1)-C3(1,1,1) - F1*D3(2,2,2))/2D0
	S4(1,3) = (C3_2(2,2,2)-C3(2,2,2) - F1*D3(3,3,3))/2D0
	S4(2,1) = (C3_3(1,1,1)+CC        - F2*D3(1,1,1))/2D0
	S4(2,2) = (           -C3(1,1,1) - F2*D3(2,2,2))/2D0
     *			-3D0*D4(0,0,2,2)
	S4(2,3) = (C3_3(2,2,2)-C3(2,2,2) - F2*D3(3,3,3))/2D0
	S4(3,1) = (C3_4(1,1,1)+CC        - F3*D3(1,1,1))/2D0
	S4(3,2) = (C3_4(2,2,2)-C3(1,1,1) - F3*D3(2,2,2))/2D0
	S4(3,3) = (           -C3(2,2,2) - F3*D3(3,3,3))/2D0
     *			-3D0*D4(0,0,3,3)

	DO 4 I=1,3
	DO 4 J=1,3
	D4(I,J,J,J) = 0D0
	DO 4 K=1,3
	D4(I,J,J,J) = D4(I,J,J,J) + YI(I,K)*S4(K,J)
4	CONTINUE

	D4(1,1,1,2) = D4(2,1,1,1)
	D4(1,1,1,3) = D4(3,1,1,1)
	D4(2,2,2,3) = D4(3,2,2,2)
	D4(2,2,2,1) = D4(1,2,2,2)

	S4X233(1) = (C3_2(1,2,2)-C3(1,2,2)-F1*D3(2,3,3))/2D0
	S4X233(2) = (           -C3(1,2,2)-F2*D3(2,3,3))/2D0
     *						-D4(0,0,3,3)
	S4X233(3) = (           -C3(1,2,2)-F3*D3(2,3,3))/2D0
     *						-2D0*D4(0,0,2,3)

	DO 41 I=1,3
	D4(I,2,3,3) = 0D0
	DO 41 K=1,3
	D4(I,2,3,3) = D4(I,2,3,3) + YI(I,K)*S4X233(K)
41	CONTINUE

	CC = C3(1,1,2)+2D0*C3(1,2,2)+C3(2,2,2)
     *			+2D0*C2(1,2)+2D0*C2(2,2)+C1(2)
	S4X113(1) = (           -CC-F1*D3(1,1,3))/2D0
     *						-2D0*D4(0,0,1,3)
	S4X113(2) = (C3_3(1,1,2)-CC-F2*D3(1,1,3))/2D0
	S4X113(3) = (           -CC-F3*D3(1,1,3))/2D0
     *						-D4(0,0,1,1)

	DO 42 I=1,3
	D4(1,1,I,3) = 0D0
	DO 42 K=1,3
	D4(1,1,I,3) = D4(1,1,I,3) + YI(I,K)*S4X113(K)
42	CONTINUE

	CC = C3(1,1,1)+C3(1,1,2)+C2(1,1)
	S4X122(1) = (            CC-F1*D3(1,2,2))/2D0
     *						-D4(0,0,2,2)
	S4X122(2) = (            CC-F2*D3(1,2,2))/2D0
     *						-2D0*D4(0,0,1,2)
	S4X122(3) = (C3_4(1,2,2)+CC-F3*D3(1,2,2))/2D0

	DO 43 I=1,3
	D4(1,2,2,I) = 0D0
	DO 43 K=1,3
	D4(1,2,2,I) = D4(1,2,2,I) + YI(I,K)*S4X122(K)
43	CONTINUE

	D4(1,1,2,2) = D4(1,2,2,1)

c*** remaining non-ordered coefficients NOT set !

	END

************************************************************************
        FUNCTION E0_(p1,p2,p3,p4,p5,s12,s23,s34,s45,s15,
     &		     m0,m1,m2,m3,m4,ext)
************************************************************************
*  	scalar 5-pt function 
*  	pi = p_i^2 = squared external momenta,  s_ij = (p_i+p_j)^2
*-----------------------------------------------------------------------
*	Reduction to five 4-pt functions as described in:
*	  A.Denner, Fortschr. Phys. 41 (1993) 307,
*	  A.Denner, S.Dittmaier and M.Roth, Nucl. Phys. B519 (1998) 39
*-----------------------------------------------------------------------
*  	24.9.98  Stefan Dittmaier	   
************************************************************************
        implicit real*8 (a-z) 
	complex*16 e0_,E0_ext,d0_,d0(0:4)
	real*8 y(0:4,0:4),dety(0:4)
	integer i,ext

	if (ext.ne.0) then
	  E0_ = E0_ext(p1,p2,p3,p4,p5,s12,s23,s34,s45,s15,
     &                 m0,m1,m2,m3,m4,ext)
	  return
	endif

	m02 = m0**2
	m12 = m1**2
	m22 = m2**2
	m32 = m3**2
	m42 = m4**2

	d0(0) = D0_( p2, p3, p4,s15,s23,s34,m1,m2,m3,m4,ext)
	d0(1) = D0_(s12, p3, p4, p5,s45,s34,m0,m2,m3,m4,ext)
	d0(2) = D0_( p1,s23, p4, p5,s45,s15,m0,m1,m3,m4,ext)
	d0(3) = D0_( p1, p2,s34, p5,s12,s15,m0,m1,m2,m4,ext)
	d0(4) = D0_( p1, p2, p3,s45,s12,s23,m0,m1,m2,m3,ext)

	y(0,0) = 2d0*m02
	y(1,1) = 2d0*m12
	y(2,2) = 2d0*m22
	y(3,3) = 2d0*m32
	y(4,4) = 2d0*m42
	y(0,1) = m02+m12-p1
	y(1,2) = m12+m22-p2
	y(2,3) = m22+m32-p3
	y(3,4) = m32+m42-p4
	y(0,4) = m42+m02-p5
	y(0,2) = m02+m22-s12
	y(1,3) = m12+m32-s23
	y(2,4) = m22+m42-s34
	y(0,3) = m32+m02-s45
	y(1,4) = m42+m12-s15

        DETYY = Y(0,4)**2*Y(1,3)**2*Y(2,2)-2D0*Y(0,3)*Y(0,4)*Y(1,3)*
     &  Y(1,4)*Y(2,2)+Y(0,3)**2*Y(1,4)**2*Y(2,2)-2D0*Y(0,4)**2*Y(1,
     &  2)*Y(1,3)*Y(2,3)+2D0*Y(0,3)*Y(0,4)*Y(1,2)*Y(1,4)*Y(2,3)+2D0*
     &  Y(0,2)*Y(0,4)*Y(1,3)*Y(1,4)*Y(2,3)-2D0*Y(0,2)*Y(0,3)*Y(1,
     &  4)**2*Y(2,3)+Y(0,4)**2*Y(1,1)*Y(2,3)**2-2D0*Y(0,1)*Y(0,4)*
     &  Y(1,4)*Y(2,3)**2+Y(0,0)*Y(1,4)**2*Y(2,3)**2+2D0*Y(0,3)*Y(0,
     &  4)*Y(1,2)*Y(1,3)*Y(2,4)-2D0*Y(0,2)*Y(0,4)*Y(1,3)**2*Y(2,4)-
     &  2D0*Y(0,3)**2*Y(1,2)*Y(1,4)*Y(2,4)+2D0*Y(0,2)*Y(0,3)*Y(1,3)*
     &  Y(1,4)*Y(2,4)-2D0*Y(0,3)*Y(0,4)*Y(1,1)*Y(2,3)*Y(2,4)+2D0*
     &  Y(0,1)*Y(0,4)*Y(1,3)*Y(2,3)*Y(2,4)+2D0*Y(0,1)*Y(0,3)*Y(1,4)*
     &  Y(2,3)*Y(2,4)-2D0*Y(0,0)*Y(1,3)*Y(1,4)*Y(2,3)*Y(2,4)+Y(0,
     &  3)**2*Y(1,1)*Y(2,4)**2-2D0*Y(0,1)*Y(0,3)*Y(1,3)*Y(2,4)**2+
     &  Y(0,0)*Y(1,3)**2*Y(2,4)**2+Y(0,4)**2*Y(1,2)**2*Y(3,3)-2D0*
     &  Y(0,2)*Y(0,4)*Y(1,2)*Y(1,4)*Y(3,3)+Y(0,2)**2*Y(1,4)**2*Y(3,
     &  3)-Y(0,4)**2*Y(1,1)*Y(2,2)*Y(3,3)+2D0*Y(0,1)*Y(0,4)*Y(1,4)*
     &  Y(2,2)*Y(3,3)-Y(0,0)*Y(1,4)**2*Y(2,2)*Y(3,3)+2D0*Y(0,2)*Y(0,
     &  4)*Y(1,1)*Y(2,4)*Y(3,3)-2D0*Y(0,1)*Y(0,4)*Y(1,2)*Y(2,4)*Y(3,
     &  3)-2D0*Y(0,1)*Y(0,2)*Y(1,4)*Y(2,4)*Y(3,3)+2D0*Y(0,0)*Y(1,2)*
     &  Y(1,4)*Y(2,4)*Y(3,3)+Y(0,1)**2*Y(2,4)**2*Y(3,3)-Y(0,0)*Y(1,
     &  1)*Y(2,4)**2*Y(3,3)-2D0*Y(0,3)*Y(0,4)*Y(1,2)**2*Y(3,4)+2D0*
     &  Y(0,2)*Y(0,4)*Y(1,2)*Y(1,3)*Y(3,4)+2D0*Y(0,2)*Y(0,3)*Y(1,2)*
     &  Y(1,4)*Y(3,4)-2D0*Y(0,2)**2*Y(1,3)*Y(1,4)*Y(3,4)+2D0*Y(0,3)*
     &  Y(0,4)*Y(1,1)*Y(2,2)*Y(3,4)-2D0*Y(0,1)*Y(0,4)*Y(1,3)*Y(2,2)*
     &  Y(3,4)-2D0*Y(0,1)*Y(0,3)*Y(1,4)*Y(2,2)*Y(3,4)+2D0*Y(0,0)*
     &  Y(1,3)*Y(1,4)*Y(2,2)*Y(3,4)-2D0*Y(0,2)*Y(0,4)*Y(1,1)*Y(2,3)*
     &  Y(3,4)+2D0*Y(0,1)*Y(0,4)*Y(1,2)*Y(2,3)*Y(3,4)+2D0*Y(0,1)*
     &  Y(0,2)*Y(1,4)*Y(2,3)*Y(3,4)-2D0*Y(0,0)*Y(1,2)*Y(1,4)*Y(2,3)*
     &  Y(3,4)-2D0*Y(0,2)*Y(0,3)*Y(1,1)*Y(2,4)*Y(3,4)+2D0*Y(0,1)*
     &  Y(0,3)*Y(1,2)*Y(2,4)*Y(3,4)+2D0*Y(0,1)*Y(0,2)*Y(1,3)*Y(2,4)*
     &  Y(3,4)-2D0*Y(0,0)*Y(1,2)*Y(1,3)*Y(2,4)*Y(3,4)-2D0*Y(0,1)**2*
     &  Y(2,3)*Y(2,4)*Y(3,4)+2D0*Y(0,0)*Y(1,1)*Y(2,3)*Y(2,4)*Y(3,4)+
     &  Y(0,2)**2*Y(1,1)*Y(3,4)**2-2D0*Y(0,1)*Y(0,2)*Y(1,2)*Y(3,4)**
     &  2+Y(0,0)*Y(1,2)**2*Y(3,4)**2+Y(0,1)**2*Y(2,2)*Y(3,4)**2-Y(0,
     &  0)*Y(1,1)*Y(2,2)*Y(3,4)**2+Y(0,3)**2*Y(1,2)**2*Y(4,4)-2D0*
     &  Y(0,2)*Y(0,3)*Y(1,2)*Y(1,3)*Y(4,4)+Y(0,2)**2*Y(1,3)**2*Y(4,
     &  4)-Y(0,3)**2*Y(1,1)*Y(2,2)*Y(4,4)+2D0*Y(0,1)*Y(0,3)*Y(1,3)*
     &  Y(2,2)*Y(4,4)-Y(0,0)*Y(1,3)**2*Y(2,2)*Y(4,4)+2D0*Y(0,2)*Y(0,
     &  3)*Y(1,1)*Y(2,3)*Y(4,4)-2D0*Y(0,1)*Y(0,3)*Y(1,2)*Y(2,3)*Y(4,
     &  4)-2D0*Y(0,1)*Y(0,2)*Y(1,3)*Y(2,3)*Y(4,4)+2D0*Y(0,0)*Y(1,2)*
     &  Y(1,3)*Y(2,3)*Y(4,4)+Y(0,1)**2*Y(2,3)**2*Y(4,4)-Y(0,0)*Y(1,
     &  1)*Y(2,3)**2*Y(4,4)-Y(0,2)**2*Y(1,1)*Y(3,3)*Y(4,4)+2D0*Y(0,
     &  1)*Y(0,2)*Y(1,2)*Y(3,3)*Y(4,4)-Y(0,0)*Y(1,2)**2*Y(3,3)*Y(4,
     &  4)-Y(0,1)**2*Y(2,2)*Y(3,3)*Y(4,4)+Y(0,0)*Y(1,1)*Y(2,2)*Y(3,
     &  3)*Y(4,4)
        DETY(0) = -(Y(0,4)*Y(1,3)**2*Y(2,2))+Y(0,3)*Y(1,3)*Y(1,4)*
     &  Y(2,2)+Y(0,4)*Y(1,3)*Y(1,4)*Y(2,2)-Y(0,3)*Y(1,4)**2*Y(2,2)+
     &  2D0*Y(0,4)*Y(1,2)*Y(1,3)*Y(2,3)-Y(0,3)*Y(1,2)*Y(1,4)*Y(2,3)-
     &  Y(0,4)*Y(1,2)*Y(1,4)*Y(2,3)-Y(0,2)*Y(1,3)*Y(1,4)*Y(2,3)-Y(0,
     &  4)*Y(1,3)*Y(1,4)*Y(2,3)+Y(0,2)*Y(1,4)**2*Y(2,3)+Y(0,3)*Y(1,
     &  4)**2*Y(2,3)-Y(0,4)*Y(1,1)*Y(2,3)**2+Y(0,1)*Y(1,4)*Y(2,3)**
     &  2+Y(0,4)*Y(1,4)*Y(2,3)**2-Y(1,4)**2*Y(2,3)**2-Y(0,3)*Y(1,2)*
     &  Y(1,3)*Y(2,4)-Y(0,4)*Y(1,2)*Y(1,3)*Y(2,4)+Y(0,2)*Y(1,3)**2*
     &  Y(2,4)+Y(0,4)*Y(1,3)**2*Y(2,4)+2D0*Y(0,3)*Y(1,2)*Y(1,4)*Y(2,
     &  4)-Y(0,2)*Y(1,3)*Y(1,4)*Y(2,4)-Y(0,3)*Y(1,3)*Y(1,4)*Y(2,4)+
     &  Y(0,3)*Y(1,1)*Y(2,3)*Y(2,4)+Y(0,4)*Y(1,1)*Y(2,3)*Y(2,4)-Y(0,
     &  1)*Y(1,3)*Y(2,3)*Y(2,4)-Y(0,4)*Y(1,3)*Y(2,3)*Y(2,4)-Y(0,1)*
     &  Y(1,4)*Y(2,3)*Y(2,4)-Y(0,3)*Y(1,4)*Y(2,3)*Y(2,4)+2D0*Y(1,3)*
     &  Y(1,4)*Y(2,3)*Y(2,4)-Y(0,3)*Y(1,1)*Y(2,4)**2+Y(0,1)*Y(1,3)*
     &  Y(2,4)**2+Y(0,3)*Y(1,3)*Y(2,4)**2-Y(1,3)**2*Y(2,4)**2-Y(0,
     &  4)*Y(1,2)**2*Y(3,3)+Y(0,2)*Y(1,2)*Y(1,4)*Y(3,3)+Y(0,4)*Y(1,
     &  2)*Y(1,4)*Y(3,3)-Y(0,2)*Y(1,4)**2*Y(3,3)+Y(0,4)*Y(1,1)*Y(2,
     &  2)*Y(3,3)-Y(0,1)*Y(1,4)*Y(2,2)*Y(3,3)-Y(0,4)*Y(1,4)*Y(2,2)*
     &  Y(3,3)+Y(1,4)**2*Y(2,2)*Y(3,3)-Y(0,2)*Y(1,1)*Y(2,4)*Y(3,3)-
     &  Y(0,4)*Y(1,1)*Y(2,4)*Y(3,3)+Y(0,1)*Y(1,2)*Y(2,4)*Y(3,3)+Y(0,
     &  4)*Y(1,2)*Y(2,4)*Y(3,3)+Y(0,1)*Y(1,4)*Y(2,4)*Y(3,3)+Y(0,2)*
     &  Y(1,4)*Y(2,4)*Y(3,3)-2D0*Y(1,2)*Y(1,4)*Y(2,4)*Y(3,3)-Y(0,1)*
     &  Y(2,4)**2*Y(3,3)+Y(1,1)*Y(2,4)**2*Y(3,3)+Y(0,3)*Y(1,2)**2*
     &  Y(3,4)+Y(0,4)*Y(1,2)**2*Y(3,4)-Y(0,2)*Y(1,2)*Y(1,3)*Y(3,4)-
     &  Y(0,4)*Y(1,2)*Y(1,3)*Y(3,4)-Y(0,2)*Y(1,2)*Y(1,4)*Y(3,4)-Y(0,
     &  3)*Y(1,2)*Y(1,4)*Y(3,4)+2D0*Y(0,2)*Y(1,3)*Y(1,4)*Y(3,4)-Y(0,
     &  3)*Y(1,1)*Y(2,2)*Y(3,4)-Y(0,4)*Y(1,1)*Y(2,2)*Y(3,4)+Y(0,1)*
     &  Y(1,3)*Y(2,2)*Y(3,4)+Y(0,4)*Y(1,3)*Y(2,2)*Y(3,4)+Y(0,1)*Y(1,
     &  4)*Y(2,2)*Y(3,4)+Y(0,3)*Y(1,4)*Y(2,2)*Y(3,4)-2D0*Y(1,3)*Y(1,
     &  4)*Y(2,2)*Y(3,4)+Y(0,2)*Y(1,1)*Y(2,3)*Y(3,4)+Y(0,4)*Y(1,1)*
     &  Y(2,3)*Y(3,4)-Y(0,1)*Y(1,2)*Y(2,3)*Y(3,4)-Y(0,4)*Y(1,2)*Y(2,
     &  3)*Y(3,4)-Y(0,1)*Y(1,4)*Y(2,3)*Y(3,4)-Y(0,2)*Y(1,4)*Y(2,3)*
     &  Y(3,4)+2D0*Y(1,2)*Y(1,4)*Y(2,3)*Y(3,4)+Y(0,2)*Y(1,1)*Y(2,4)*
     &  Y(3,4)+Y(0,3)*Y(1,1)*Y(2,4)*Y(3,4)-Y(0,1)*Y(1,2)*Y(2,4)*Y(3,
     &  4)-Y(0,3)*Y(1,2)*Y(2,4)*Y(3,4)-Y(0,1)*Y(1,3)*Y(2,4)*Y(3,4)-
     &  Y(0,2)*Y(1,3)*Y(2,4)*Y(3,4)+2D0*Y(1,2)*Y(1,3)*Y(2,4)*Y(3,4)+
     &  2D0*Y(0,1)*Y(2,3)*Y(2,4)*Y(3,4)-2D0*Y(1,1)*Y(2,3)*Y(2,4)*
     &  Y(3,4)-Y(0,2)*Y(1,1)*Y(3,4)**2+Y(0,1)*Y(1,2)*Y(3,4)**2+Y(0,
     &  2)*Y(1,2)*Y(3,4)**2-Y(1,2)**2*Y(3,4)**2-Y(0,1)*Y(2,2)*Y(3,
     &  4)**2+Y(1,1)*Y(2,2)*Y(3,4)**2-Y(0,3)*Y(1,2)**2*Y(4,4)+Y(0,
     &  2)*Y(1,2)*Y(1,3)*Y(4,4)+Y(0,3)*Y(1,2)*Y(1,3)*Y(4,4)-Y(0,2)*
     &  Y(1,3)**2*Y(4,4)+Y(0,3)*Y(1,1)*Y(2,2)*Y(4,4)-Y(0,1)*Y(1,3)*
     &  Y(2,2)*Y(4,4)-Y(0,3)*Y(1,3)*Y(2,2)*Y(4,4)+Y(1,3)**2*Y(2,2)*
     &  Y(4,4)-Y(0,2)*Y(1,1)*Y(2,3)*Y(4,4)-Y(0,3)*Y(1,1)*Y(2,3)*Y(4,
     &  4)+Y(0,1)*Y(1,2)*Y(2,3)*Y(4,4)+Y(0,3)*Y(1,2)*Y(2,3)*Y(4,4)+
     &  Y(0,1)*Y(1,3)*Y(2,3)*Y(4,4)+Y(0,2)*Y(1,3)*Y(2,3)*Y(4,4)-2D0*
     &  Y(1,2)*Y(1,3)*Y(2,3)*Y(4,4)-Y(0,1)*Y(2,3)**2*Y(4,4)+Y(1,1)*
     &  Y(2,3)**2*Y(4,4)+Y(0,2)*Y(1,1)*Y(3,3)*Y(4,4)-Y(0,1)*Y(1,2)*
     &  Y(3,3)*Y(4,4)-Y(0,2)*Y(1,2)*Y(3,3)*Y(4,4)+Y(1,2)**2*Y(3,3)*
     &  Y(4,4)+Y(0,1)*Y(2,2)*Y(3,3)*Y(4,4)-Y(1,1)*Y(2,2)*Y(3,3)*Y(4,
     &  4)
        DETY(1) = Y(0,3)*Y(0,4)*Y(1,3)*Y(2,2)-Y(0,4)**2*Y(1,3)*Y(2,
     &  2)-Y(0,3)**2*Y(1,4)*Y(2,2)+Y(0,3)*Y(0,4)*Y(1,4)*Y(2,2)-Y(0,
     &  3)*Y(0,4)*Y(1,2)*Y(2,3)+Y(0,4)**2*Y(1,2)*Y(2,3)-Y(0,2)*Y(0,
     &  4)*Y(1,3)*Y(2,3)+Y(0,4)**2*Y(1,3)*Y(2,3)+2D0*Y(0,2)*Y(0,3)*
     &  Y(1,4)*Y(2,3)-Y(0,2)*Y(0,4)*Y(1,4)*Y(2,3)-Y(0,3)*Y(0,4)*Y(1,
     &  4)*Y(2,3)+Y(0,1)*Y(0,4)*Y(2,3)**2-Y(0,4)**2*Y(2,3)**2-Y(0,
     &  0)*Y(1,4)*Y(2,3)**2+Y(0,4)*Y(1,4)*Y(2,3)**2+Y(0,3)**2*Y(1,
     &  2)*Y(2,4)-Y(0,3)*Y(0,4)*Y(1,2)*Y(2,4)-Y(0,2)*Y(0,3)*Y(1,3)*
     &  Y(2,4)+2D0*Y(0,2)*Y(0,4)*Y(1,3)*Y(2,4)-Y(0,3)*Y(0,4)*Y(1,3)*
     &  Y(2,4)-Y(0,2)*Y(0,3)*Y(1,4)*Y(2,4)+Y(0,3)**2*Y(1,4)*Y(2,4)-
     &  Y(0,1)*Y(0,3)*Y(2,3)*Y(2,4)-Y(0,1)*Y(0,4)*Y(2,3)*Y(2,4)+2D0*
     &  Y(0,3)*Y(0,4)*Y(2,3)*Y(2,4)+Y(0,0)*Y(1,3)*Y(2,3)*Y(2,4)-Y(0,
     &  4)*Y(1,3)*Y(2,3)*Y(2,4)+Y(0,0)*Y(1,4)*Y(2,3)*Y(2,4)-Y(0,3)*
     &  Y(1,4)*Y(2,3)*Y(2,4)+Y(0,1)*Y(0,3)*Y(2,4)**2-Y(0,3)**2*Y(2,
     &  4)**2-Y(0,0)*Y(1,3)*Y(2,4)**2+Y(0,3)*Y(1,3)*Y(2,4)**2+Y(0,
     &  2)*Y(0,4)*Y(1,2)*Y(3,3)-Y(0,4)**2*Y(1,2)*Y(3,3)-Y(0,2)**2*
     &  Y(1,4)*Y(3,3)+Y(0,2)*Y(0,4)*Y(1,4)*Y(3,3)-Y(0,1)*Y(0,4)*Y(2,
     &  2)*Y(3,3)+Y(0,4)**2*Y(2,2)*Y(3,3)+Y(0,0)*Y(1,4)*Y(2,2)*Y(3,
     &  3)-Y(0,4)*Y(1,4)*Y(2,2)*Y(3,3)+Y(0,1)*Y(0,2)*Y(2,4)*Y(3,3)+
     &  Y(0,1)*Y(0,4)*Y(2,4)*Y(3,3)-2D0*Y(0,2)*Y(0,4)*Y(2,4)*Y(3,3)-
     &  Y(0,0)*Y(1,2)*Y(2,4)*Y(3,3)+Y(0,4)*Y(1,2)*Y(2,4)*Y(3,3)-Y(0,
     &  0)*Y(1,4)*Y(2,4)*Y(3,3)+Y(0,2)*Y(1,4)*Y(2,4)*Y(3,3)+Y(0,0)*
     &  Y(2,4)**2*Y(3,3)-Y(0,1)*Y(2,4)**2*Y(3,3)-Y(0,2)*Y(0,3)*Y(1,
     &  2)*Y(3,4)-Y(0,2)*Y(0,4)*Y(1,2)*Y(3,4)+2D0*Y(0,3)*Y(0,4)*Y(1,
     &  2)*Y(3,4)+Y(0,2)**2*Y(1,3)*Y(3,4)-Y(0,2)*Y(0,4)*Y(1,3)*Y(3,
     &  4)+Y(0,2)**2*Y(1,4)*Y(3,4)-Y(0,2)*Y(0,3)*Y(1,4)*Y(3,4)+Y(0,
     &  1)*Y(0,3)*Y(2,2)*Y(3,4)+Y(0,1)*Y(0,4)*Y(2,2)*Y(3,4)-2D0*Y(0,
     &  3)*Y(0,4)*Y(2,2)*Y(3,4)-Y(0,0)*Y(1,3)*Y(2,2)*Y(3,4)+Y(0,4)*
     &  Y(1,3)*Y(2,2)*Y(3,4)-Y(0,0)*Y(1,4)*Y(2,2)*Y(3,4)+Y(0,3)*Y(1,
     &  4)*Y(2,2)*Y(3,4)-Y(0,1)*Y(0,2)*Y(2,3)*Y(3,4)-Y(0,1)*Y(0,4)*
     &  Y(2,3)*Y(3,4)+2D0*Y(0,2)*Y(0,4)*Y(2,3)*Y(3,4)+Y(0,0)*Y(1,2)*
     &  Y(2,3)*Y(3,4)-Y(0,4)*Y(1,2)*Y(2,3)*Y(3,4)+Y(0,0)*Y(1,4)*Y(2,
     &  3)*Y(3,4)-Y(0,2)*Y(1,4)*Y(2,3)*Y(3,4)-Y(0,1)*Y(0,2)*Y(2,4)*
     &  Y(3,4)-Y(0,1)*Y(0,3)*Y(2,4)*Y(3,4)+2D0*Y(0,2)*Y(0,3)*Y(2,4)*
     &  Y(3,4)+Y(0,0)*Y(1,2)*Y(2,4)*Y(3,4)-Y(0,3)*Y(1,2)*Y(2,4)*Y(3,
     &  4)+Y(0,0)*Y(1,3)*Y(2,4)*Y(3,4)-Y(0,2)*Y(1,3)*Y(2,4)*Y(3,4)-
     &  2D0*Y(0,0)*Y(2,3)*Y(2,4)*Y(3,4)+2D0*Y(0,1)*Y(2,3)*Y(2,4)*
     &  Y(3,4)+Y(0,1)*Y(0,2)*Y(3,4)**2-Y(0,2)**2*Y(3,4)**2-Y(0,0)*
     &  Y(1,2)*Y(3,4)**2+Y(0,2)*Y(1,2)*Y(3,4)**2+Y(0,0)*Y(2,2)*Y(3,
     &  4)**2-Y(0,1)*Y(2,2)*Y(3,4)**2+Y(0,2)*Y(0,3)*Y(1,2)*Y(4,4)-
     &  Y(0,3)**2*Y(1,2)*Y(4,4)-Y(0,2)**2*Y(1,3)*Y(4,4)+Y(0,2)*Y(0,
     &  3)*Y(1,3)*Y(4,4)-Y(0,1)*Y(0,3)*Y(2,2)*Y(4,4)+Y(0,3)**2*Y(2,
     &  2)*Y(4,4)+Y(0,0)*Y(1,3)*Y(2,2)*Y(4,4)-Y(0,3)*Y(1,3)*Y(2,2)*
     &  Y(4,4)+Y(0,1)*Y(0,2)*Y(2,3)*Y(4,4)+Y(0,1)*Y(0,3)*Y(2,3)*Y(4,
     &  4)-2D0*Y(0,2)*Y(0,3)*Y(2,3)*Y(4,4)-Y(0,0)*Y(1,2)*Y(2,3)*Y(4,
     &  4)+Y(0,3)*Y(1,2)*Y(2,3)*Y(4,4)-Y(0,0)*Y(1,3)*Y(2,3)*Y(4,4)+
     &  Y(0,2)*Y(1,3)*Y(2,3)*Y(4,4)+Y(0,0)*Y(2,3)**2*Y(4,4)-Y(0,1)*
     &  Y(2,3)**2*Y(4,4)-Y(0,1)*Y(0,2)*Y(3,3)*Y(4,4)+Y(0,2)**2*Y(3,
     &  3)*Y(4,4)+Y(0,0)*Y(1,2)*Y(3,3)*Y(4,4)-Y(0,2)*Y(1,2)*Y(3,3)*
     &  Y(4,4)-Y(0,0)*Y(2,2)*Y(3,3)*Y(4,4)+Y(0,1)*Y(2,2)*Y(3,3)*Y(4,
     &  4)
        DETY(2) = -(Y(0,3)*Y(0,4)*Y(1,2)*Y(1,3))+Y(0,4)**2*Y(1,2)*
     &  Y(1,3)+Y(0,2)*Y(0,4)*Y(1,3)**2-Y(0,4)**2*Y(1,3)**2+Y(0,3)**
     &  2*Y(1,2)*Y(1,4)-Y(0,3)*Y(0,4)*Y(1,2)*Y(1,4)-Y(0,2)*Y(0,3)*
     &  Y(1,3)*Y(1,4)-Y(0,2)*Y(0,4)*Y(1,3)*Y(1,4)+2D0*Y(0,3)*Y(0,4)*
     &  Y(1,3)*Y(1,4)+Y(0,2)*Y(0,3)*Y(1,4)**2-Y(0,3)**2*Y(1,4)**2+
     &  Y(0,3)*Y(0,4)*Y(1,1)*Y(2,3)-Y(0,4)**2*Y(1,1)*Y(2,3)-Y(0,1)*
     &  Y(0,4)*Y(1,3)*Y(2,3)+Y(0,4)**2*Y(1,3)*Y(2,3)-Y(0,1)*Y(0,3)*
     &  Y(1,4)*Y(2,3)+2D0*Y(0,1)*Y(0,4)*Y(1,4)*Y(2,3)-Y(0,3)*Y(0,4)*
     &  Y(1,4)*Y(2,3)+Y(0,0)*Y(1,3)*Y(1,4)*Y(2,3)-Y(0,4)*Y(1,3)*Y(1,
     &  4)*Y(2,3)-Y(0,0)*Y(1,4)**2*Y(2,3)+Y(0,3)*Y(1,4)**2*Y(2,3)-
     &  Y(0,3)**2*Y(1,1)*Y(2,4)+Y(0,3)*Y(0,4)*Y(1,1)*Y(2,4)+2D0*Y(0,
     &  1)*Y(0,3)*Y(1,3)*Y(2,4)-Y(0,1)*Y(0,4)*Y(1,3)*Y(2,4)-Y(0,3)*
     &  Y(0,4)*Y(1,3)*Y(2,4)-Y(0,0)*Y(1,3)**2*Y(2,4)+Y(0,4)*Y(1,3)**
     &  2*Y(2,4)-Y(0,1)*Y(0,3)*Y(1,4)*Y(2,4)+Y(0,3)**2*Y(1,4)*Y(2,
     &  4)+Y(0,0)*Y(1,3)*Y(1,4)*Y(2,4)-Y(0,3)*Y(1,3)*Y(1,4)*Y(2,4)-
     &  Y(0,2)*Y(0,4)*Y(1,1)*Y(3,3)+Y(0,4)**2*Y(1,1)*Y(3,3)+Y(0,1)*
     &  Y(0,4)*Y(1,2)*Y(3,3)-Y(0,4)**2*Y(1,2)*Y(3,3)+Y(0,1)*Y(0,2)*
     &  Y(1,4)*Y(3,3)-2D0*Y(0,1)*Y(0,4)*Y(1,4)*Y(3,3)+Y(0,2)*Y(0,4)*
     &  Y(1,4)*Y(3,3)-Y(0,0)*Y(1,2)*Y(1,4)*Y(3,3)+Y(0,4)*Y(1,2)*Y(1,
     &  4)*Y(3,3)+Y(0,0)*Y(1,4)**2*Y(3,3)-Y(0,2)*Y(1,4)**2*Y(3,3)-
     &  Y(0,1)**2*Y(2,4)*Y(3,3)+Y(0,1)*Y(0,4)*Y(2,4)*Y(3,3)+Y(0,0)*
     &  Y(1,1)*Y(2,4)*Y(3,3)-Y(0,4)*Y(1,1)*Y(2,4)*Y(3,3)-Y(0,0)*Y(1,
     &  4)*Y(2,4)*Y(3,3)+Y(0,1)*Y(1,4)*Y(2,4)*Y(3,3)+Y(0,2)*Y(0,3)*
     &  Y(1,1)*Y(3,4)+Y(0,2)*Y(0,4)*Y(1,1)*Y(3,4)-2D0*Y(0,3)*Y(0,4)*
     &  Y(1,1)*Y(3,4)-Y(0,1)*Y(0,3)*Y(1,2)*Y(3,4)-Y(0,1)*Y(0,4)*Y(1,
     &  2)*Y(3,4)+2D0*Y(0,3)*Y(0,4)*Y(1,2)*Y(3,4)-Y(0,1)*Y(0,2)*Y(1,
     &  3)*Y(3,4)+2D0*Y(0,1)*Y(0,4)*Y(1,3)*Y(3,4)-Y(0,2)*Y(0,4)*Y(1,
     &  3)*Y(3,4)+Y(0,0)*Y(1,2)*Y(1,3)*Y(3,4)-Y(0,4)*Y(1,2)*Y(1,3)*
     &  Y(3,4)-Y(0,1)*Y(0,2)*Y(1,4)*Y(3,4)+2D0*Y(0,1)*Y(0,3)*Y(1,4)*
     &  Y(3,4)-Y(0,2)*Y(0,3)*Y(1,4)*Y(3,4)+Y(0,0)*Y(1,2)*Y(1,4)*Y(3,
     &  4)-Y(0,3)*Y(1,2)*Y(1,4)*Y(3,4)-2D0*Y(0,0)*Y(1,3)*Y(1,4)*Y(3,
     &  4)+2D0*Y(0,2)*Y(1,3)*Y(1,4)*Y(3,4)+Y(0,1)**2*Y(2,3)*Y(3,4)-
     &  Y(0,1)*Y(0,4)*Y(2,3)*Y(3,4)-Y(0,0)*Y(1,1)*Y(2,3)*Y(3,4)+Y(0,
     &  4)*Y(1,1)*Y(2,3)*Y(3,4)+Y(0,0)*Y(1,4)*Y(2,3)*Y(3,4)-Y(0,1)*
     &  Y(1,4)*Y(2,3)*Y(3,4)+Y(0,1)**2*Y(2,4)*Y(3,4)-Y(0,1)*Y(0,3)*
     &  Y(2,4)*Y(3,4)-Y(0,0)*Y(1,1)*Y(2,4)*Y(3,4)+Y(0,3)*Y(1,1)*Y(2,
     &  4)*Y(3,4)+Y(0,0)*Y(1,3)*Y(2,4)*Y(3,4)-Y(0,1)*Y(1,3)*Y(2,4)*
     &  Y(3,4)-Y(0,1)**2*Y(3,4)**2+Y(0,1)*Y(0,2)*Y(3,4)**2+Y(0,0)*
     &  Y(1,1)*Y(3,4)**2-Y(0,2)*Y(1,1)*Y(3,4)**2-Y(0,0)*Y(1,2)*Y(3,
     &  4)**2+Y(0,1)*Y(1,2)*Y(3,4)**2-Y(0,2)*Y(0,3)*Y(1,1)*Y(4,4)+
     &  Y(0,3)**2*Y(1,1)*Y(4,4)+Y(0,1)*Y(0,3)*Y(1,2)*Y(4,4)-Y(0,3)**
     &  2*Y(1,2)*Y(4,4)+Y(0,1)*Y(0,2)*Y(1,3)*Y(4,4)-2D0*Y(0,1)*Y(0,
     &  3)*Y(1,3)*Y(4,4)+Y(0,2)*Y(0,3)*Y(1,3)*Y(4,4)-Y(0,0)*Y(1,2)*
     &  Y(1,3)*Y(4,4)+Y(0,3)*Y(1,2)*Y(1,3)*Y(4,4)+Y(0,0)*Y(1,3)**2*
     &  Y(4,4)-Y(0,2)*Y(1,3)**2*Y(4,4)-Y(0,1)**2*Y(2,3)*Y(4,4)+Y(0,
     &  1)*Y(0,3)*Y(2,3)*Y(4,4)+Y(0,0)*Y(1,1)*Y(2,3)*Y(4,4)-Y(0,3)*
     &  Y(1,1)*Y(2,3)*Y(4,4)-Y(0,0)*Y(1,3)*Y(2,3)*Y(4,4)+Y(0,1)*Y(1,
     &  3)*Y(2,3)*Y(4,4)+Y(0,1)**2*Y(3,3)*Y(4,4)-Y(0,1)*Y(0,2)*Y(3,
     &  3)*Y(4,4)-Y(0,0)*Y(1,1)*Y(3,3)*Y(4,4)+Y(0,2)*Y(1,1)*Y(3,3)*
     &  Y(4,4)+Y(0,0)*Y(1,2)*Y(3,3)*Y(4,4)-Y(0,1)*Y(1,2)*Y(3,3)*Y(4,
     &  4)
        DETY(3) = Y(0,3)*Y(0,4)*Y(1,2)**2-Y(0,4)**2*Y(1,2)**2-Y(0,
     &  2)*Y(0,4)*Y(1,2)*Y(1,3)+Y(0,4)**2*Y(1,2)*Y(1,3)-Y(0,2)*Y(0,
     &  3)*Y(1,2)*Y(1,4)+2D0*Y(0,2)*Y(0,4)*Y(1,2)*Y(1,4)-Y(0,3)*Y(0,
     &  4)*Y(1,2)*Y(1,4)+Y(0,2)**2*Y(1,3)*Y(1,4)-Y(0,2)*Y(0,4)*Y(1,
     &  3)*Y(1,4)-Y(0,2)**2*Y(1,4)**2+Y(0,2)*Y(0,3)*Y(1,4)**2-Y(0,
     &  3)*Y(0,4)*Y(1,1)*Y(2,2)+Y(0,4)**2*Y(1,1)*Y(2,2)+Y(0,1)*Y(0,
     &  4)*Y(1,3)*Y(2,2)-Y(0,4)**2*Y(1,3)*Y(2,2)+Y(0,1)*Y(0,3)*Y(1,
     &  4)*Y(2,2)-2D0*Y(0,1)*Y(0,4)*Y(1,4)*Y(2,2)+Y(0,3)*Y(0,4)*Y(1,
     &  4)*Y(2,2)-Y(0,0)*Y(1,3)*Y(1,4)*Y(2,2)+Y(0,4)*Y(1,3)*Y(1,4)*
     &  Y(2,2)+Y(0,0)*Y(1,4)**2*Y(2,2)-Y(0,3)*Y(1,4)**2*Y(2,2)+Y(0,
     &  2)*Y(0,4)*Y(1,1)*Y(2,3)-Y(0,4)**2*Y(1,1)*Y(2,3)-Y(0,1)*Y(0,
     &  4)*Y(1,2)*Y(2,3)+Y(0,4)**2*Y(1,2)*Y(2,3)-Y(0,1)*Y(0,2)*Y(1,
     &  4)*Y(2,3)+2D0*Y(0,1)*Y(0,4)*Y(1,4)*Y(2,3)-Y(0,2)*Y(0,4)*Y(1,
     &  4)*Y(2,3)+Y(0,0)*Y(1,2)*Y(1,4)*Y(2,3)-Y(0,4)*Y(1,2)*Y(1,4)*
     &  Y(2,3)-Y(0,0)*Y(1,4)**2*Y(2,3)+Y(0,2)*Y(1,4)**2*Y(2,3)+Y(0,
     &  2)*Y(0,3)*Y(1,1)*Y(2,4)-2D0*Y(0,2)*Y(0,4)*Y(1,1)*Y(2,4)+Y(0,
     &  3)*Y(0,4)*Y(1,1)*Y(2,4)-Y(0,1)*Y(0,3)*Y(1,2)*Y(2,4)+2D0*Y(0,
     &  1)*Y(0,4)*Y(1,2)*Y(2,4)-Y(0,3)*Y(0,4)*Y(1,2)*Y(2,4)-Y(0,1)*
     &  Y(0,2)*Y(1,3)*Y(2,4)-Y(0,1)*Y(0,4)*Y(1,3)*Y(2,4)+2D0*Y(0,2)*
     &  Y(0,4)*Y(1,3)*Y(2,4)+Y(0,0)*Y(1,2)*Y(1,3)*Y(2,4)-Y(0,4)*Y(1,
     &  2)*Y(1,3)*Y(2,4)+2D0*Y(0,1)*Y(0,2)*Y(1,4)*Y(2,4)-Y(0,1)*Y(0,
     &  3)*Y(1,4)*Y(2,4)-Y(0,2)*Y(0,3)*Y(1,4)*Y(2,4)-2D0*Y(0,0)*Y(1,
     &  2)*Y(1,4)*Y(2,4)+2D0*Y(0,3)*Y(1,2)*Y(1,4)*Y(2,4)+Y(0,0)*Y(1,
     &  3)*Y(1,4)*Y(2,4)-Y(0,2)*Y(1,3)*Y(1,4)*Y(2,4)+Y(0,1)**2*Y(2,
     &  3)*Y(2,4)-Y(0,1)*Y(0,4)*Y(2,3)*Y(2,4)-Y(0,0)*Y(1,1)*Y(2,3)*
     &  Y(2,4)+Y(0,4)*Y(1,1)*Y(2,3)*Y(2,4)+Y(0,0)*Y(1,4)*Y(2,3)*Y(2,
     &  4)-Y(0,1)*Y(1,4)*Y(2,3)*Y(2,4)-Y(0,1)**2*Y(2,4)**2+Y(0,1)*
     &  Y(0,3)*Y(2,4)**2+Y(0,0)*Y(1,1)*Y(2,4)**2-Y(0,3)*Y(1,1)*Y(2,
     &  4)**2-Y(0,0)*Y(1,3)*Y(2,4)**2+Y(0,1)*Y(1,3)*Y(2,4)**2-Y(0,
     &  2)**2*Y(1,1)*Y(3,4)+Y(0,2)*Y(0,4)*Y(1,1)*Y(3,4)+2D0*Y(0,1)*
     &  Y(0,2)*Y(1,2)*Y(3,4)-Y(0,1)*Y(0,4)*Y(1,2)*Y(3,4)-Y(0,2)*Y(0,
     &  4)*Y(1,2)*Y(3,4)-Y(0,0)*Y(1,2)**2*Y(3,4)+Y(0,4)*Y(1,2)**2*
     &  Y(3,4)-Y(0,1)*Y(0,2)*Y(1,4)*Y(3,4)+Y(0,2)**2*Y(1,4)*Y(3,4)+
     &  Y(0,0)*Y(1,2)*Y(1,4)*Y(3,4)-Y(0,2)*Y(1,2)*Y(1,4)*Y(3,4)-Y(0,
     &  1)**2*Y(2,2)*Y(3,4)+Y(0,1)*Y(0,4)*Y(2,2)*Y(3,4)+Y(0,0)*Y(1,
     &  1)*Y(2,2)*Y(3,4)-Y(0,4)*Y(1,1)*Y(2,2)*Y(3,4)-Y(0,0)*Y(1,4)*
     &  Y(2,2)*Y(3,4)+Y(0,1)*Y(1,4)*Y(2,2)*Y(3,4)+Y(0,1)**2*Y(2,4)*
     &  Y(3,4)-Y(0,1)*Y(0,2)*Y(2,4)*Y(3,4)-Y(0,0)*Y(1,1)*Y(2,4)*Y(3,
     &  4)+Y(0,2)*Y(1,1)*Y(2,4)*Y(3,4)+Y(0,0)*Y(1,2)*Y(2,4)*Y(3,4)-
     &  Y(0,1)*Y(1,2)*Y(2,4)*Y(3,4)+Y(0,2)**2*Y(1,1)*Y(4,4)-Y(0,2)*
     &  Y(0,3)*Y(1,1)*Y(4,4)-2D0*Y(0,1)*Y(0,2)*Y(1,2)*Y(4,4)+Y(0,1)*
     &  Y(0,3)*Y(1,2)*Y(4,4)+Y(0,2)*Y(0,3)*Y(1,2)*Y(4,4)+Y(0,0)*Y(1,
     &  2)**2*Y(4,4)-Y(0,3)*Y(1,2)**2*Y(4,4)+Y(0,1)*Y(0,2)*Y(1,3)*
     &  Y(4,4)-Y(0,2)**2*Y(1,3)*Y(4,4)-Y(0,0)*Y(1,2)*Y(1,3)*Y(4,4)+
     &  Y(0,2)*Y(1,2)*Y(1,3)*Y(4,4)+Y(0,1)**2*Y(2,2)*Y(4,4)-Y(0,1)*
     &  Y(0,3)*Y(2,2)*Y(4,4)-Y(0,0)*Y(1,1)*Y(2,2)*Y(4,4)+Y(0,3)*Y(1,
     &  1)*Y(2,2)*Y(4,4)+Y(0,0)*Y(1,3)*Y(2,2)*Y(4,4)-Y(0,1)*Y(1,3)*
     &  Y(2,2)*Y(4,4)-Y(0,1)**2*Y(2,3)*Y(4,4)+Y(0,1)*Y(0,2)*Y(2,3)*
     &  Y(4,4)+Y(0,0)*Y(1,1)*Y(2,3)*Y(4,4)-Y(0,2)*Y(1,1)*Y(2,3)*Y(4,
     &  4)-Y(0,0)*Y(1,2)*Y(2,3)*Y(4,4)+Y(0,1)*Y(1,2)*Y(2,3)*Y(4,4)
        DETY(4) = -(Y(0,3)**2*Y(1,2)**2)+Y(0,3)*Y(0,4)*Y(1,2)**2+
     &  2D0*Y(0,2)*Y(0,3)*Y(1,2)*Y(1,3)-Y(0,2)*Y(0,4)*Y(1,2)*Y(1,3)-
     &  Y(0,3)*Y(0,4)*Y(1,2)*Y(1,3)-Y(0,2)**2*Y(1,3)**2+Y(0,2)*Y(0,
     &  4)*Y(1,3)**2-Y(0,2)*Y(0,3)*Y(1,2)*Y(1,4)+Y(0,3)**2*Y(1,2)*
     &  Y(1,4)+Y(0,2)**2*Y(1,3)*Y(1,4)-Y(0,2)*Y(0,3)*Y(1,3)*Y(1,4)+
     &  Y(0,3)**2*Y(1,1)*Y(2,2)-Y(0,3)*Y(0,4)*Y(1,1)*Y(2,2)-2D0*Y(0,
     &  1)*Y(0,3)*Y(1,3)*Y(2,2)+Y(0,1)*Y(0,4)*Y(1,3)*Y(2,2)+Y(0,3)*
     &  Y(0,4)*Y(1,3)*Y(2,2)+Y(0,0)*Y(1,3)**2*Y(2,2)-Y(0,4)*Y(1,3)**
     &  2*Y(2,2)+Y(0,1)*Y(0,3)*Y(1,4)*Y(2,2)-Y(0,3)**2*Y(1,4)*Y(2,
     &  2)-Y(0,0)*Y(1,3)*Y(1,4)*Y(2,2)+Y(0,3)*Y(1,3)*Y(1,4)*Y(2,2)-
     &  2D0*Y(0,2)*Y(0,3)*Y(1,1)*Y(2,3)+Y(0,2)*Y(0,4)*Y(1,1)*Y(2,3)+
     &  Y(0,3)*Y(0,4)*Y(1,1)*Y(2,3)+2D0*Y(0,1)*Y(0,3)*Y(1,2)*Y(2,3)-
     &  Y(0,1)*Y(0,4)*Y(1,2)*Y(2,3)-Y(0,3)*Y(0,4)*Y(1,2)*Y(2,3)+2D0*
     &  Y(0,1)*Y(0,2)*Y(1,3)*Y(2,3)-Y(0,1)*Y(0,4)*Y(1,3)*Y(2,3)-Y(0,
     &  2)*Y(0,4)*Y(1,3)*Y(2,3)-2D0*Y(0,0)*Y(1,2)*Y(1,3)*Y(2,3)+2D0*
     &  Y(0,4)*Y(1,2)*Y(1,3)*Y(2,3)-Y(0,1)*Y(0,2)*Y(1,4)*Y(2,3)-Y(0,
     &  1)*Y(0,3)*Y(1,4)*Y(2,3)+2D0*Y(0,2)*Y(0,3)*Y(1,4)*Y(2,3)+Y(0,
     &  0)*Y(1,2)*Y(1,4)*Y(2,3)-Y(0,3)*Y(1,2)*Y(1,4)*Y(2,3)+Y(0,0)*
     &  Y(1,3)*Y(1,4)*Y(2,3)-Y(0,2)*Y(1,3)*Y(1,4)*Y(2,3)-Y(0,1)**2*
     &  Y(2,3)**2+Y(0,1)*Y(0,4)*Y(2,3)**2+Y(0,0)*Y(1,1)*Y(2,3)**2-
     &  Y(0,4)*Y(1,1)*Y(2,3)**2-Y(0,0)*Y(1,4)*Y(2,3)**2+Y(0,1)*Y(1,
     &  4)*Y(2,3)**2+Y(0,2)*Y(0,3)*Y(1,1)*Y(2,4)-Y(0,3)**2*Y(1,1)*
     &  Y(2,4)-Y(0,1)*Y(0,3)*Y(1,2)*Y(2,4)+Y(0,3)**2*Y(1,2)*Y(2,4)-
     &  Y(0,1)*Y(0,2)*Y(1,3)*Y(2,4)+2D0*Y(0,1)*Y(0,3)*Y(1,3)*Y(2,4)-
     &  Y(0,2)*Y(0,3)*Y(1,3)*Y(2,4)+Y(0,0)*Y(1,2)*Y(1,3)*Y(2,4)-Y(0,
     &  3)*Y(1,2)*Y(1,3)*Y(2,4)-Y(0,0)*Y(1,3)**2*Y(2,4)+Y(0,2)*Y(1,
     &  3)**2*Y(2,4)+Y(0,1)**2*Y(2,3)*Y(2,4)-Y(0,1)*Y(0,3)*Y(2,3)*
     &  Y(2,4)-Y(0,0)*Y(1,1)*Y(2,3)*Y(2,4)+Y(0,3)*Y(1,1)*Y(2,3)*Y(2,
     &  4)+Y(0,0)*Y(1,3)*Y(2,3)*Y(2,4)-Y(0,1)*Y(1,3)*Y(2,3)*Y(2,4)+
     &  Y(0,2)**2*Y(1,1)*Y(3,3)-Y(0,2)*Y(0,4)*Y(1,1)*Y(3,3)-2D0*Y(0,
     &  1)*Y(0,2)*Y(1,2)*Y(3,3)+Y(0,1)*Y(0,4)*Y(1,2)*Y(3,3)+Y(0,2)*
     &  Y(0,4)*Y(1,2)*Y(3,3)+Y(0,0)*Y(1,2)**2*Y(3,3)-Y(0,4)*Y(1,2)**
     &  2*Y(3,3)+Y(0,1)*Y(0,2)*Y(1,4)*Y(3,3)-Y(0,2)**2*Y(1,4)*Y(3,
     &  3)-Y(0,0)*Y(1,2)*Y(1,4)*Y(3,3)+Y(0,2)*Y(1,2)*Y(1,4)*Y(3,3)+
     &  Y(0,1)**2*Y(2,2)*Y(3,3)-Y(0,1)*Y(0,4)*Y(2,2)*Y(3,3)-Y(0,0)*
     &  Y(1,1)*Y(2,2)*Y(3,3)+Y(0,4)*Y(1,1)*Y(2,2)*Y(3,3)+Y(0,0)*Y(1,
     &  4)*Y(2,2)*Y(3,3)-Y(0,1)*Y(1,4)*Y(2,2)*Y(3,3)-Y(0,1)**2*Y(2,
     &  4)*Y(3,3)+Y(0,1)*Y(0,2)*Y(2,4)*Y(3,3)+Y(0,0)*Y(1,1)*Y(2,4)*
     &  Y(3,3)-Y(0,2)*Y(1,1)*Y(2,4)*Y(3,3)-Y(0,0)*Y(1,2)*Y(2,4)*Y(3,
     &  3)+Y(0,1)*Y(1,2)*Y(2,4)*Y(3,3)-Y(0,2)**2*Y(1,1)*Y(3,4)+Y(0,
     &  2)*Y(0,3)*Y(1,1)*Y(3,4)+2D0*Y(0,1)*Y(0,2)*Y(1,2)*Y(3,4)-Y(0,
     &  1)*Y(0,3)*Y(1,2)*Y(3,4)-Y(0,2)*Y(0,3)*Y(1,2)*Y(3,4)-Y(0,0)*
     &  Y(1,2)**2*Y(3,4)+Y(0,3)*Y(1,2)**2*Y(3,4)-Y(0,1)*Y(0,2)*Y(1,
     &  3)*Y(3,4)+Y(0,2)**2*Y(1,3)*Y(3,4)+Y(0,0)*Y(1,2)*Y(1,3)*Y(3,
     &  4)-Y(0,2)*Y(1,2)*Y(1,3)*Y(3,4)-Y(0,1)**2*Y(2,2)*Y(3,4)+Y(0,
     &  1)*Y(0,3)*Y(2,2)*Y(3,4)+Y(0,0)*Y(1,1)*Y(2,2)*Y(3,4)-Y(0,3)*
     &  Y(1,1)*Y(2,2)*Y(3,4)-Y(0,0)*Y(1,3)*Y(2,2)*Y(3,4)+Y(0,1)*Y(1,
     &  3)*Y(2,2)*Y(3,4)+Y(0,1)**2*Y(2,3)*Y(3,4)-Y(0,1)*Y(0,2)*Y(2,
     &  3)*Y(3,4)-Y(0,0)*Y(1,1)*Y(2,3)*Y(3,4)+Y(0,2)*Y(1,1)*Y(2,3)*
     &  Y(3,4)+Y(0,0)*Y(1,2)*Y(2,3)*Y(3,4)-Y(0,1)*Y(1,2)*Y(2,3)*Y(3,
     &  4)

	E0_ = 0d0
	do 10 i=0,4
	E0_ = E0_ + D0(i)*dety(i)
10	continue
	E0_ = E0_/detyy
 
	end

***********************************************************************
	SUBROUTINE E01234_(p1,p2,p3,p4,p5,s12,s23,s34,s45,s15,
     &		m0,m1,m2,m3,m4,E0,E1,E2,E3,E4,D0,D1,D2,D3,rank,ext)
***********************************************************************
*	5-POINT TENSOR COEFFICIENTS
*	COMPLEX VALUED
*----------------------------------------------------------------------
*	24.10.98 Stefan Dittmaier
***********************************************************************
        IMPLICIT REAL*8 (A-Z)                                         
        COMPLEX*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 D0,D1(3),D2(0:3,0:3),D3(0:3,0:3,3)
	COMPLEX*16 D4(0:3,0:3,0:3,0:3)
        COMPLEX*16 D0_1,D1_1(3),D2_1(0:3,0:3),D3_1(0:3,0:3,3)
	COMPLEX*16 D4_1(0:3,0:3,0:3,0:3)
        COMPLEX*16 D0_2,D1_2(3),D2_2(0:3,0:3),D3_2(0:3,0:3,3)
	COMPLEX*16 D4_2(0:3,0:3,0:3,0:3)
        COMPLEX*16 D0_3,D1_3(3),D2_3(0:3,0:3),D3_3(0:3,0:3,3)
	COMPLEX*16 D4_3(0:3,0:3,0:3,0:3)
        COMPLEX*16 D0_4,D1_4(3),D2_4(0:3,0:3),D3_4(0:3,0:3,3)
	COMPLEX*16 D4_4(0:3,0:3,0:3,0:3)
        COMPLEX*16 E0_,E0,E1(4),E2(4,4),E3(4,4,4),E4(4,4,4,4)
        COMPLEX*16 T1(4),T2(4,4),T3(4,4,4),T4(4,4,4,4)
        COMPLEX*16 D1sum,D2sum0,D2sum(3),D3sum00,D3sum0(3),D3sum(3,3)
        COMPLEX*16 D00diff(4),D00kdiff(4,4)
	REAL*8 f(4),ZI(4,4)
	INTEGER j,k,l,m,n,rank,ext

        E0 = E0_(p1,p2,p3,p4,p5,s12,s23,s34,s45,s15,m0,m1,m2,m3,m4,ext)
	if (rank.eq.0) return

	p1p1 = 2d0* p1
	p2p2 = 2d0*s12
	p3p3 = 2d0*s45
	p4p4 = 2d0* p5
	p1p2 = s12+ p1- p2
	p2p3 = s45+s12- p3
	p3p4 = s45+ p5- p4
	p1p3 =  p1+s45-s23
	p2p4 =  p5+s12-s34
	p1p4 =  p1+ p5-s15

	f(1) =  p1-m1**2+m0**2
	f(2) = s12-m2**2+m0**2
	f(3) = s45-m3**2+m0**2
	f(4) =  p5-m4**2+m0**2

        ZDET = P1P4**2*P2P3**2-2D0*P1P3*P1P4*P2P3*P2P4+P1P3**2*
     &  P2P4**2-P1P4**2*P2P2*P3P3+2D0*P1P2*P1P4*P2P4*P3P3-P1P1*
     &  P2P4**2*P3P3+2D0*P1P3*P1P4*P2P2*P3P4-2D0*P1P2*P1P4*P2P3*
     &  P3P4-2D0*P1P2*P1P3*P2P4*P3P4+2D0*P1P1*P2P3*P2P4*P3P4+P1P2**
     &  2*P3P4**2-P1P1*P2P2*P3P4**2-P1P3**2*P2P2*P4P4+2D0*P1P2*P1P3*
     &  P2P3*P4P4-P1P1*P2P3**2*P4P4-P1P2**2*P3P3*P4P4+P1P1*P2P2*
     &  P3P3*P4P4
        ZI(1,1) = (-P2P4**2*P3P3+2D0*P2P3*P2P4*P3P4-P2P2*P3P4**2-
     &  	   P2P3**2*P4P4+P2P2*P3P3*P4P4)/ZDET
        ZI(1,2) = (P1P4*P2P4*P3P3-P1P4*P2P3*P3P4-P1P3*P2P4*P3P4+
     &  	   P1P2*P3P4**2+P1P3*P2P3*P4P4-P1P2*P3P3*P4P4)/ZDET
        ZI(1,3) = (-P1P4*P2P3*P2P4+P1P3*P2P4**2+P1P4*P2P2*P3P4-
     &  	   P1P2*P2P4*P3P4-P1P3*P2P2*P4P4+P1P2*P2P3*P4P4)/ZDET
        ZI(1,4) = (P1P4*P2P3**2-P1P3*P2P3*P2P4-P1P4*P2P2*P3P3+P1P2*
     &  	   P2P4*P3P3+P1P3*P2P2*P3P4-P1P2*P2P3*P3P4)/ZDET
        ZI(2,2) = (-P1P4**2*P3P3+2D0*P1P3*P1P4*P3P4-P1P1*P3P4**2-
     &  	   P1P3**2*P4P4+P1P1*P3P3*P4P4)/ZDET
        ZI(2,3) = (P1P4**2*P2P3-P1P3*P1P4*P2P4-P1P2*P1P4*P3P4+P1P1*
     &  	   P2P4*P3P4+P1P2*P1P3*P4P4-P1P1*P2P3*P4P4)/ZDET
        ZI(2,4) = (-P1P3*P1P4*P2P3+P1P3**2*P2P4+P1P2*P1P4*P3P3-
     &  	   P1P1*P2P4*P3P3-P1P2*P1P3*P3P4+P1P1*P2P3*P3P4)/ZDET
        ZI(3,3) = (-P1P4**2*P2P2+2D0*P1P2*P1P4*P2P4-P1P1*P2P4**2-
     &  	   P1P2**2*P4P4+P1P1*P2P2*P4P4)/ZDET
        ZI(3,4) = (P1P3*P1P4*P2P2-P1P2*P1P4*P2P3-P1P2*P1P3*P2P4+
     &  	   P1P1*P2P3*P2P4+P1P2**2*P3P4-P1P1*P2P2*P3P4)/ZDET
        ZI(4,4) = (-P1P3**2*P2P2+2D0*P1P2*P1P3*P2P3-P1P1*P2P3**2-
     &  	   P1P2**2*P3P3+P1P1*P2P2*P3P3)/ZDET
        ZI(2,1) = ZI(1,2)
        ZI(3,1) = ZI(1,3)
        ZI(3,2) = ZI(2,3)
        ZI(4,1) = ZI(1,4)
        ZI(4,2) = ZI(2,4)
        ZI(4,3) = ZI(3,4)

c*** Note order of arguments 5 and 6 in D01234_ !
	call D01234_( p2, p3, p4,s15,s34,s23,m1,m2,m3,m4,
     *		   D0,D1,D2,D3,D4,C0,C1,C2,C3,rank-1,ext)
 	call D01234_(s12, p3, p4, p5,s34,s45,m0,m2,m3,m4,
     *		   D0_1,D1_1,D2_1,D3_1,D4_1,C0,C1,C2,C3,rank-1,ext)
	call D01234_( p1,s23, p4, p5,s15,s45,m0,m1,m3,m4,
     *		   D0_2,D1_2,D2_2,D3_2,D4_2,C0,C1,C2,C3,rank-1,ext)
	call D01234_( p1, p2,s34, p5,s15,s12,m0,m1,m2,m4,
     *		   D0_3,D1_3,D2_3,D3_3,D4_3,C0,C1,C2,C3,rank-1,ext)
	call D01234_( p1, p2, p3,s45,s23,s12,m0,m1,m2,m3,
     *		   D0_4,D1_4,D2_4,D3_4,D4_4,C0,C1,C2,C3,rank-1,ext)

	T1(1) = D0_1-D0
	T1(2) = D0_2-D0
	T1(3) = D0_3-D0
	T1(4) = D0_4-D0

	do 1 k=1,4
	E1(k) = 0d0
	do 1 j=1,4
	E1(k) = E1(k) + ZI(k,j)*(T1(j)-f(j)*E0)
1	continue

	if (rank.eq.1) return

	D1sum = D0 + D1(1) + D1(2) + D1(3) 

	T2(1,1) =         + D1sum
	T2(1,2) = D1_1(1) - D1(1)
	T2(1,3) = D1_1(2) - D1(2) 
	T2(1,4) = D1_1(3) - D1(3) 

	T2(2,1) = D1_2(1) + D1sum 
	T2(2,2) =         - D1(1) 
	T2(2,3) = D1_2(2) - D1(2) 
	T2(2,4) = D1_2(3) - D1(3) 

	T2(3,1) = D1_3(1) + D1sum
	T2(3,2) = D1_3(2) - D1(1)
	T2(3,3) =         - D1(2)
	T2(3,4) = D1_3(3) - D1(3)

	T2(4,1) = D1_4(1) + D1sum 
	T2(4,2) = D1_4(2) - D1(1) 
	T2(4,3) = D1_4(3) - D1(2) 
	T2(4,4) =         - D1(3) 

	do 2 k=1,4
	do 2 l=1,4
	E2(k,l) = 0d0
	do 2 j=1,4
	E2(k,l) = E2(k,l) + ZI(k,j)*(T2(j,l)-f(j)*E1(l))
2	continue

	if (rank.eq.2) return

	D2sum0   = D2(1,1)+D2(2,2)+D2(3,3)+2d0*(D2(1,2)+D2(1,3)+D2(2,3))
     &		   +2d0*(D1(1)+D1(2)+D1(3))+D0
	D2sum(1) = D2(1,1)+D2(1,2)+D2(1,3)+D1(1)
	D2sum(2) = D2(1,2)+D2(2,2)+D2(2,3)+D1(2)
	D2sum(3) = D2(1,3)+D2(2,3)+D2(3,3)+D1(3)

	D00diff(1) = D2_1(0,0) - D2(0,0)
	D00diff(2) = D2_2(0,0) - D2(0,0)
	D00diff(3) = D2_3(0,0) - D2(0,0)
	D00diff(4) = D2_4(0,0) - D2(0,0)

	T3(1,1,1) =           - D2sum0
	T3(1,1,2) =             D2sum(1) 
	T3(1,1,3) =             D2sum(2) 
	T3(1,1,4) =             D2sum(3) 
	T3(1,2,2) = D2_1(1,1) - D2(1,1)  
	T3(1,2,3) = D2_1(1,2) - D2(1,2)  
	T3(1,2,4) = D2_1(1,3) - D2(1,3)  
	T3(1,3,3) = D2_1(2,2) - D2(2,2)  
	T3(1,3,4) = D2_1(2,3) - D2(2,3)  
	T3(1,4,4) = D2_1(3,3) - D2(3,3)  
 
	T3(2,1,1) = D2_2(1,1) - D2sum0 
	T3(2,1,2) =             D2sum(1) 
	T3(2,1,3) = D2_2(1,2) + D2sum(2) 
	T3(2,1,4) = D2_2(1,3) + D2sum(3) 
	T3(2,2,2) =           - D2(1,1)  
	T3(2,2,3) =           - D2(1,2)  
	T3(2,2,4) =           - D2(1,3)  
	T3(2,3,3) = D2_2(2,2) - D2(2,2)  
	T3(2,3,4) = D2_2(2,3) - D2(2,3)  
	T3(2,4,4) = D2_2(3,3) - D2(3,3)  

	T3(3,1,1) = D2_3(1,1) - D2sum0 
	T3(3,1,2) = D2_3(1,2) + D2sum(1) 
	T3(3,1,3) =             D2sum(2) 
	T3(3,1,4) = D2_3(1,3) + D2sum(3) 
	T3(3,2,2) = D2_3(2,2) - D2(1,1)  
	T3(3,2,3) =           - D2(1,2)  
	T3(3,2,4) = D2_3(2,3) - D2(1,3)  
	T3(3,3,3) =           - D2(2,2)  
	T3(3,3,4) =           - D2(2,3)  
	T3(3,4,4) = D2_3(3,3) - D2(3,3)  
 
	T3(4,1,1) = D2_4(1,1) - D2sum0 
	T3(4,1,2) = D2_4(1,2) + D2sum(1) 
	T3(4,1,3) = D2_4(1,3) + D2sum(2) 
	T3(4,1,4) =             D2sum(3) 
	T3(4,2,2) = D2_4(2,2) - D2(1,1)  
	T3(4,2,3) = D2_4(2,3) - D2(1,2)  
	T3(4,2,4) =           - D2(1,3)  
	T3(4,3,3) = D2_4(3,3) - D2(2,2)  
	T3(4,3,4) =           - D2(2,3)  
	T3(4,4,4) =           - D2(3,3)  
 
	do 30 j=1,4
	do 30 l=1,4
	do 30 m=1,l-1
	T3(j,l,m) = T3(j,m,l) 
30	continue

	do 3 k=1,4
	do 3 l=1,4
	do 3 m=1,4
	E3(k,l,m) = 0d0
	do 3 j=1,4
	E3(k,l,m) = E3(k,l,m) + ZI(k,j)*( T3(j,l,m)
     &		+ 2d0*ZI(l,m)*D00diff(j) - f(j)*E2(l,m) )
3	continue

	if (rank.eq.3) return

	D3sum00    =              D0          + 6d0*ZI(1,1)*D2(0,0) 
	do 41 l=1,3
	D3sum00    = D3sum00    + 3d0*D1(l)   + 6d0*ZI(1,1)*D3(0,0,l)
	D3sum0(l)  =              D1(l)       - 4d0*ZI(1,l+1)*D2(0,0)
	do 41 m=1,3
	D3sum00    = D3sum00    + 3d0*D2(l,m)
	D3sum0(l)  = D3sum0(l)  + 2d0*D2(l,m) - 4d0*ZI(1,l+1)*D3(0,0,m)
	D3sum(l,m) =              D2(l,m)     + 2d0*ZI(l+1,m+1)*D2(0,0) 
	do 41 n=1,3
	D3sum00    = D3sum00    + D3(l,m,n) 
	D3sum0(l)  = D3sum0(l)  + D3(l,m,n) 
	D3sum(l,m) = D3sum(l,m) + D3(l,m,n) + 2d0*ZI(l+1,m+1)*D3(0,0,n)
41	continue

	D00kdiff(1,1) = 0d0
	D00kdiff(1,2) = D3_1(0,0,1) - D3(0,0,1)
	D00kdiff(1,3) = D3_1(0,0,2) - D3(0,0,2)
	D00kdiff(1,4) = D3_1(0,0,3) - D3(0,0,3)

	D00kdiff(2,1) = D3_2(0,0,1)
	D00kdiff(2,2) =             - D3(0,0,1)
	D00kdiff(2,3) = D3_2(0,0,2) - D3(0,0,2)
	D00kdiff(2,4) = D3_2(0,0,3) - D3(0,0,3)

	D00kdiff(3,1) = D3_3(0,0,1)
	D00kdiff(3,2) = D3_3(0,0,2) - D3(0,0,1)
	D00kdiff(3,3) =             - D3(0,0,2)
	D00kdiff(3,4) = D3_3(0,0,3) - D3(0,0,3)

	D00kdiff(4,1) = D3_4(0,0,1)
	D00kdiff(4,2) = D3_4(0,0,2) - D3(0,0,1)
	D00kdiff(4,3) = D3_4(0,0,3) - D3(0,0,2)
	D00kdiff(4,4) =             - D3(0,0,3)

	T4(1,1,1,1) =               D3sum00
	T4(1,1,1,2) =             - D3sum0(1)
	T4(1,1,1,3) =             - D3sum0(2)
	T4(1,1,1,4) =             - D3sum0(3)
	T4(1,1,2,2) =               D3sum(1,1)
	T4(1,1,2,3) =               D3sum(1,2)
	T4(1,1,2,4) =               D3sum(1,3)
	T4(1,1,3,3) =               D3sum(2,2)
	T4(1,1,3,4) =               D3sum(2,3)
	T4(1,1,4,4) =               D3sum(3,3)
	T4(1,2,2,2) = D3_1(1,1,1) - D3(1,1,1) 
	T4(1,2,2,3) = D3_1(1,1,2) - D3(1,1,2) 
	T4(1,2,2,4) = D3_1(1,1,3) - D3(1,1,3) 
	T4(1,2,3,3) = D3_1(1,2,2) - D3(1,2,2) 
	T4(1,2,3,4) = D3_1(1,2,3) - D3(1,2,3) 
	T4(1,2,4,4) = D3_1(1,3,3) - D3(1,3,3) 
	T4(1,3,3,3) = D3_1(2,2,2) - D3(2,2,2) 
	T4(1,3,3,4) = D3_1(2,2,3) - D3(2,2,3) 
	T4(1,3,4,4) = D3_1(2,3,3) - D3(2,3,3) 
	T4(1,4,4,4) = D3_1(3,3,3) - D3(3,3,3) 

	T4(2,1,1,1) = D3_2(1,1,1) + D3sum00
	T4(2,1,1,2) =             - D3sum0(1)
	T4(2,1,1,3) = D3_2(1,1,2) - D3sum0(2)
	T4(2,1,1,4) = D3_2(1,1,3) - D3sum0(3)
	T4(2,1,2,2) =               D3sum(1,1)
	T4(2,1,2,3) =               D3sum(1,2)
	T4(2,1,2,4) =               D3sum(1,3)
	T4(2,1,3,3) = D3_2(1,2,2) + D3sum(2,2)
	T4(2,1,3,4) = D3_2(1,2,3) + D3sum(2,3)
	T4(2,1,4,4) = D3_2(1,3,3) + D3sum(3,3)
	T4(2,2,2,2) =             - D3(1,1,1) 
	T4(2,2,2,3) =             - D3(1,1,2) 
	T4(2,2,2,4) =             - D3(1,1,3) 
	T4(2,2,3,3) =             - D3(1,2,2) 
	T4(2,2,3,4) =             - D3(1,2,3) 
	T4(2,2,4,4) =             - D3(1,3,3) 
	T4(2,3,3,3) = D3_2(2,2,2) - D3(2,2,2) 
	T4(2,3,3,4) = D3_2(2,2,3) - D3(2,2,3) 
	T4(2,3,4,4) = D3_2(2,3,3) - D3(2,3,3) 
	T4(2,4,4,4) = D3_2(3,3,3) - D3(3,3,3) 

	T4(3,1,1,1) = D3_3(1,1,1) + D3sum00
	T4(3,1,1,2) = D3_3(1,1,2) - D3sum0(1)
	T4(3,1,1,3) =             - D3sum0(2)
	T4(3,1,1,4) = D3_3(1,1,3) - D3sum0(3)
	T4(3,1,2,2) = D3_3(1,2,2) + D3sum(1,1)
	T4(3,1,2,3) =               D3sum(1,2)
	T4(3,1,2,4) = D3_3(1,2,3) + D3sum(1,3)
	T4(3,1,3,3) =               D3sum(2,2)
	T4(3,1,3,4) =               D3sum(2,3)
	T4(3,1,4,4) = D3_3(1,3,3) + D3sum(3,3)
	T4(3,2,2,2) = D3_3(2,2,2) - D3(1,1,1) 
	T4(3,2,2,3) =             - D3(1,1,2) 
	T4(3,2,2,4) = D3_3(2,2,3) - D3(1,1,3) 
	T4(3,2,3,3) =             - D3(1,2,2) 
	T4(3,2,3,4) =             - D3(1,2,3) 
	T4(3,2,4,4) = D3_3(2,3,3) - D3(1,3,3) 
	T4(3,3,3,3) =             - D3(2,2,2) 
	T4(3,3,3,4) =             - D3(2,2,3) 
	T4(3,3,4,4) =             - D3(2,3,3) 
	T4(3,4,4,4) = D3_3(3,3,3) - D3(3,3,3) 

	T4(4,1,1,1) = D3_4(1,1,1) + D3sum00
	T4(4,1,1,2) = D3_4(1,1,2) - D3sum0(1)
	T4(4,1,1,3) = D3_4(1,1,3) - D3sum0(2)
	T4(4,1,1,4) =             - D3sum0(3)
	T4(4,1,2,2) = D3_4(1,2,2) + D3sum(1,1)
	T4(4,1,2,3) = D3_4(1,2,3) + D3sum(1,2)
	T4(4,1,2,4) =               D3sum(1,3)
	T4(4,1,3,3) = D3_4(1,3,3) + D3sum(2,2)
	T4(4,1,3,4) =               D3sum(2,3)
	T4(4,1,4,4) =               D3sum(3,3)
	T4(4,2,2,2) = D3_4(2,2,2) - D3(1,1,1) 
	T4(4,2,2,3) = D3_4(2,2,3) - D3(1,1,2) 
	T4(4,2,2,4) =             - D3(1,1,3) 
	T4(4,2,3,3) = D3_4(2,3,3) - D3(1,2,2) 
	T4(4,2,3,4) =             - D3(1,2,3) 
	T4(4,2,4,4) =             - D3(1,3,3) 
	T4(4,3,3,3) = D3_4(3,3,3) - D3(2,2,2) 
	T4(4,3,3,4) =             - D3(2,2,3) 
	T4(4,3,4,4) =             - D3(2,3,3) 
	T4(4,4,4,4) =             - D3(3,3,3) 

	do 40 j=1,4
	do 40 l=1,4
	do 40 m=1,l-1
	T4(j,l,l,m) = T4(j,m,l,l) 
	T4(j,l,m,l) = T4(j,m,l,l) 
	T4(j,m,l,m) = T4(j,m,m,l) 
	T4(j,l,m,m) = T4(j,m,m,l) 
	do 40 n=1,m-1
	T4(j,n,l,m) = T4(j,n,m,l) 
	T4(j,l,m,n) = T4(j,n,m,l) 
	T4(j,l,n,m) = T4(j,n,m,l) 
	T4(j,m,l,n) = T4(j,n,m,l) 
	T4(j,m,n,l) = T4(j,n,m,l) 
40	continue

	do 4 k=1,4
	do 4 l=1,4
	do 4 m=1,4
	do 4 n=1,4
	E4(k,l,m,n) = 0d0
	do 4 j=1,4
	E4(k,l,m,n) = E4(k,l,m,n) + ZI(k,j)*( T4(j,l,m,n)
     &		+ 2d0*ZI(m,n)*D00kdiff(j,l) + 2d0*ZI(l,n)*D00kdiff(j,m)
     &		+ 2d0*ZI(l,m)*D00kdiff(j,n) - f(j)*E3(l,m,n)	)
4	continue

	END

