c********************************************************************
      program RacoonWW 
c********************************************************************
c
c     Monte Carlo program for e+ e- -> 4 fermions (+ gamma) 
c
c Authors:  A.Denner, S.Dittmaier, M.Roth, D.Wackeroth
c
c to be submitted to Comp. Phys. Comm.
c
c additional publications:
c Nucl. Phys. B587 (2000) (hep-ph/0006307)
c Nucl. Phys. B560 (1999) (hep-ph/9904472) 
c
c      Main program
c     --------------
c
c contacts: roth@itp.uni-leipzig.de, dow@pas.rochester.edu
c           ansgar.denner@psi.ch, stefan.dittmaier@cern.ch  
c********************************************************************
c
c  VERSION 1.0 
c
c  released on January 24, 2001  
c********************************************************************
      implicit none
c local variables
      integer maxe
      parameter(maxe=29)
      real*8 energy,energy2,average,sigma,conv,rej(maxe),alphahelp
      real*8 random(15),rand,y,ecutp,ecutl,ecutq,scutqq,ccutpb,scale
      real*8 ccutpl,ccutpq,ccutll,ccutqq,ccutlq,ccutlb,ccutqb
      real*8 brlep,brhad,brtot,wwidth,av,si,avb,sigb
      real*8 weightmax(maxe),weighttotmax,pu(7,0:3)
      real*8 averagemax,kern,dscinti(maxe),gwl,gwq,pwwidth,nqcdfac
      character*6 gfname(0:1,-1:1,3),dgfname(-1:1)
      character*90 outputfile,process
      integer n,nout,nevents,i1,i2,i3,i4,ng,nl,steps,qpaw
      integer nunweighted(-1:1),novershoot,nweight(maxe),ninit,sunw
c pol 
      real*8 pm,pp
c processpar
      real*8 mv(3),mv2(3),gv(3),qcdfac,qcdfac2,q(6)
      complex*16 cvf(-1:1,3,6),cvw(2)
      integer qprop,quark(6),iso(6),gen(6),dir(7),pol(7),color
      character*6 fname(6),dfname(6)
c param
      real*8 pi,el,alpha,alpha0,alphaz,alphas,gf,cw,cw2,sw,sw2,mw,mw2
      real*8 gw,mz,mz2,gz,mh,mh2,mxe,mxe2,ml(3),ml2(3),mqp(3)
      real*8 mqp2(3),mqm(3),mqm2(3)
c processtype
      integer type,nc,cc,sym,perm
c cuts
      real*8 ecut(7),scut(7,7),ccut(7,7),precomb(4),precombg(2)
      integer scuts,srecomb,srecombg
c options
      integer smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem
      integer ssigepem4,ssigepem5,ssigepemg5,siba
c event 
      real*8 weight(maxe),pbeam(2,0:3),p(7,0:3,maxe)
      real*8 weightborn,pborn(7,0:3)
      integer nsubevent
c uv
      real*8 mudim2
c ir
      real*8 lambda,lambda2
c rcoptions
      integer qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
c gwrcoptions
      integer qgwborn,qgwqcd,qgwbos,qgwferm
c rscheme
      real*8 dalpz,drbos,drferm
      integer qalp,qgw
c maxweight
      real*8 wmax(7),wgmax,wqmax
c par_conti
      real*8 conti(maxe),conti_tot(maxe),dconti_tot(maxe)
c par_n
      integer ntot(2),ncut(2),ncut1_k(2),ncut2_k(2),ncutq_r(2)
      integer ncut3_k(2),ncut4_k(2),ncut5_k(2),ncut6_k(2),ncut_r(2)
c subcontr
      real*8 subav(4),subsi(4),submax(4)
      integer subch(4),subn(4),subrej(maxe)
c wswitches
      integer wflag(maxe)
c common blocks
      common/pol/pm,pp
      common/processpar/cvf,cvw,mv,mv2,gv,qcdfac,qcdfac2,
     *  q,qprop,quark,iso,gen,dir,pol,color,fname,dfname
      common/param/pi,el,alpha,alpha0,alphaz,alphas,gf,cw,cw2,sw,sw2,
     *  mw,mw2,gw,mz,mz2,gz,mh,mh2,mxe,mxe2,ml,ml2,mqp,mqp2,mqm,mqm2
      common/processtype/type,nc,cc,sym,perm
      common/cuts/ecut,scut,ccut,precomb,precombg,scuts,
     $     srecomb,srecombg
      common/options/smc,sborn4,sborn5,sborng5,sisr,src,son,sqcdepem,
     *  ssigepem4,ssigepem5,ssigepemg5,siba
      common/event/weight,pbeam,p,weightborn,pborn,nsubevent
      common/uv/mudim2
      common/ir/lambda,lambda2
      common/rcoptions/qborn,qreal,qqcd,qbos,qferm,qsoft,qnf,qfast
      common/gwrcoptions/qgwborn,qgwqcd,qgwbos,qgwferm
      common/rscheme/dalpz,drbos,drferm,qalp,qgw
      common/maxweight/wmax,wgmax,wqmax
      common/par_conti/conti,conti_tot,dconti_tot
      common/par_n/ntot,ncut,ncut1_k,ncut2_k,ncut3_k,ncut4_k,
     *  ncut5_k,ncut6_k,ncut_r,ncutq_r
      common/subcontr/subav,subsi,submax,subch,subn,subrej
      common/wswitches/wflag
c initialize arrays for cuts 
      do i1=1,7
         ecut(i1)=0d0
         do i2=1,7
            scut(i1,i2)=0d0
            ccut(i1,i2)=0d0
         end do
      end do
      do i1=1,4
         precomb(i1)=0d0
      end do
      do i1=1,2
         precombg(i1)=0d0
      end do
      ecutp=0d0
      ecutl=0d0
      ecutq=0d0
      scutqq=0d0
      ccutpb=0d0
      ccutpl=0d0
      ccutpq=0d0
      ccutll=0d0
      ccutqq=0d0
      ccutlq=0d0
      ccutlb=0d0
      ccutqb=0d0
c initialize array for final-state polarizations
      do i1=3,7
         pol(i1)=0d0
      end do
c initialize array for wswitches 
c a priori all sub-contributions are included (1)
c (not included: 0)
c wflag(maxe) are only relevant for smc=3 !
      do i1=1,maxe
         wflag(i1)=1
      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     read input file                                             c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      read'(a)',outputfile     ! name of output file
      read*,energy             ! CMF energy
      read*,nevents            ! number of events
      read*,smc                ! choice of MC branch 
      read*,sborn4             ! include Born ee->4f
      read*,sborn5             ! include Born ee->4fa
      read*,sborng5            ! include Born ee->4fg
      read*,siba               ! IBA
      read*,sisr               ! include higher-order ISR
      read*,src                ! include radiative corrections
      read*,qnf                ! choice of Coulomb singularity
      read*,qreal              ! neglect imaginary part of virtual corr.
      read*,qalp               ! choice of input-parameter scheme
      read*,qgw                ! calculate W-boson width
      read*,qprop              ! choice of width scheme
      read*,ssigepem4          ! choice of diagrams for Born ee->4f
      read*,ssigepem5          ! choice of diagrams for Born ee->4fa
      read*,ssigepemg5         ! choice of diagrams for Born ee->4fg
      read*,qqcd               ! include QCD radiative corrections
      read*,sqcdepem           ! include gluon-exch. diagrams in Born
      do i1=3,6
        read'(a)',fname(i1)    ! particle names
      enddo
      read*,pp                 ! degree of positron beam polarization
      read*,pm                 ! degree of electron beam polarization
      read*,srecomb            ! recombination cuts
      read*,precomb(1)         ! angular rec. cut between photon and beam
      read*,precomb(2)         ! rec. cut on photon energy
      read*,precomb(3)         ! invariant-mass rec. (TH) or
c                              ! angular rec. cut for leptons (EXP)
      read*,precomb(4)         ! angular rec. cut for quarks (EXP)
      read*,srecombg           ! gluon recombination cuts
      read*,precombg(1)        ! rec. cut on gluon energy
      read*,precombg(2)        ! invariant-mass (TH) or angular (EXP) rec.
      read*,scuts              ! separation cuts
      if(scuts.eq.10)then
c simplified input
        read*,ecutp            ! photon(gluon) energy cut
        read*,ecutl            ! charged-lepton energy cut
        read*,ecutq            ! quark energy cut
        read*,scutqq           ! quark-quark invariant mass cut
        read*,ccutpb           ! angular cut between photon and beam
        read*,ccutpl           ! angular cut between photon and charged lepton
        read*,ccutpq           ! angular cut between photon(gluon) and quark
        read*,ccutll           ! angular cut between charged leptons
        read*,ccutqq           ! angular cut between quarks
        read*,ccutlq           ! angular cut between charged lepton and quark
        read*,ccutlb           ! angular cut between charged lepton and beam
        read*,ccutqb           ! angular cut between quark and beam
      elseif(scuts.eq.11)then
c most general input
        do i1=3,7
          read*,ecut(i1)       ! energy cuts
        enddo
        do i1=1,7
        do i2=max(3,i1+1),7
          read*,ccut(i1,i2)    ! angular cuts
        enddo
        enddo
        do i1=3,7
        do i2=i1+1,7
          read*,scut(i1,i2)    ! invariant-mass cuts
        enddo
        enddo
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting parameters                                          c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      i2=0
      do i1=1,90
        if(outputfile(i1:i1).eq.' ')i2=1
        if(i2.eq.1)outputfile(i1:i1)=' '
      enddo
      if(outputfile(1:1).eq.' ')then
        nout=6 
      else
        nout=22
        open(unit=nout,file=outputfile,status='unknown')
      endif
      open(unit=30,file='optimization.info',status='unknown')
c setting input parameters
      pi=4d0*datan(1d0)
      call parameter
c further parameters
      conv=3.8937966d11       ! conversion factor GeV^-2 to fbarns
      mxe=1d-15               ! small electron mass for tensor reduction
      lambda=1d0              ! photon mass
      mudim2=1d0              ! renormalization scale
      scale=energy            ! scale in structure functions for ISR
c alternatively, scale^2=-t_min(on-shell):
c      scale=dsqrt(dabs(mw**2-energy**2/2d0*
c     $     (1d0-dsqrt(1d0-4d0*mw**2/energy**2))))
c calculating parameters
      cw=mw/mz                ! weak mixing angle
      cw2=cw**2
      sw2=1d0-cw2
      sw=sqrt(sw2)
      mw2=mw**2
      mz2=mz**2
      mh2=mh**2
      mxe2=mxe**2
      lambda2=lambda**2
      do i1=1,3
        ml2(i1)=ml(i1)**2
        mqp2(i1)=mqp(i1)**2
        mqm2(i1)=mqm(i1)**2
      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c warnings !
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(energy.eq.0d0)then
        write(*,'(a)')' Zero CMF energy is not allowed!'
        stop
      end if
      if(src.eq.1)then
        if(energy.eq.2d0*mz)then
          write(*,'(a)')' Energy = 2 M_Z: change energy!'
          stop
        endif
        if(energy.eq.2d0*mqp(3))then
          write(*,'(a)')' Energy = 2 m_top: change energy!'
          stop
        endif
        if(energy.eq.mh+mz)then
          write(*,'(a)')' Energy = M_H + M_Z: change energy!'
          stop
        endif
        if(energy.eq.2d0*mw)then
          write(*,'(a)')' Energy <= 2 M_W: change energy!'
          stop
        endif
      endif
      if(nevents.eq.0)then
        write(*,'(a)')' Zero number of events is not allowed!'
        stop
      endif
      if(smc.eq.0.or.smc.gt.3)then
         write(*,'(a)')' Please specify MC branch: smc=1,2 or 3!'
         stop
      endif
      if(src.eq.0.and.sborn5.eq.1.and.sborn4.ge.1)then
         write(*,'(a)')' Please choose sborn5=1 OR sborn4>0!'
         stop
      endif
      if(src.eq.0.and.sborng5.eq.1.and.sborn4.ge.1)then
         write(*,'(a)')' Please choose sborng5=1 OR sborn4>0!'
         stop
      endif
      if(src.eq.0.and.sborn5.eq.1.and.sborng5.eq.1)then
         write(*,'(a)')' Please choose sborn5=1 OR sborng5=1!'
         stop
      endif
      if(src.eq.0.and.sborn5.eq.1.and.scuts.eq.0)then
         write(*,'(a)')' Please apply separation cuts on the photon!'
         stop
      endif
      if(src.eq.0.and.sborng5.eq.1.and.scuts.eq.0)then
         write(*,'(a)')' Please apply separation cuts on the gluon!'
         stop
      endif
      if(src.ne.0.and.srecomb.ne.0.and.srecomb.ne.3.and.
     $     precomb(1).eq.0d0)then
         write(*,'(a)')' precomb(1)=0d0 is not allowed !'
         stop
      endif
      if(src.ne.0.and.srecomb.ne.0.and.precomb(2).eq.0d0)then
         write(*,'(a)')' precomb(2)=0d0 is not allowed !'
         stop
      endif
      if(src.ne.0.and.srecomb.ne.0.and.precomb(3).eq.0d0)then
         write(*,'(a)')' precomb(3)=0d0 is not allowed !'
         stop
      endif
      if(src.ne.0.and.srecomb.eq.2.and.precomb(4).eq.0d0)then
         write(*,'(a)')' precomb(4)=0d0 is not allowed !'
         stop
      endif
      if(qqcd.eq.1.or.qqcd.eq.3)then
         write(*,'(a)')' qqcd=1,3 is not yet supported !'
         stop
      end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c For smc=3 only the following choices are allowed:
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(smc.eq.3)then
        if(src.ne.1.or.sborn4.ne.1.or.pp.ne.0d0.or.pm.ne.0d0)then
          write(*,'(a)')'warning for smc=3:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See NOTE.1 in README.'
          src=1
          sborn4=1
          pp=0d0
          pm=0d0
        endif
      endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c For IBA only the following choices are allowed:                
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(siba.ne.0)then
        if(src.ne.0.or.sborn4.ne.0.or.sborn5.ne.0.or.sisr.ne.1.or.
     *    qprop.ne.1.or.qqcd.eq.1.or.qqcd.eq.3.or.
     $    qalp.ne.2.or.sborng5.ne.0.or.(qgw.ne.0.and.qgw.ne.2))then
          write(*,'(a)')'warning for IBA:' 
          write(*,'(a)')'RacoonWW overwrites some of your choices,'
          write(*,'(a)')'since not allowed. See NOTE.2 in README.'
        end if
        src=0
        sborn4=0
        sborn5=0
        sborng5=0
        sisr=1
        qprop=1
        if(qqcd.ne.0)qqcd=2
        qalp=2
        if(qgw.ne.0.and.qgw.ne.2)qgw=2
      end if
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     parametrization scheme                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(qalp.eq.0)then
	alpha=alpha0
      elseif(qalp.eq.1)then
	alpha=alphaz
      elseif(qalp.eq.2)then
	alpha=sqrt(2d0)*gf/pi*mw2*sw2
      endif
      el=sqrt(4d0*pi*alpha)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     calculating the total W-boson width                         c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(qgw.ne.0)then
        call calcrenc
        call calcrenc_ferm
        qgwborn=1
        alphahelp=alpha
c tree-level W width
        if(qgw.eq.1)then
          qgwqcd=0
          qgwbos=0
          qgwferm=0
c IBA W width
        elseif(qgw.eq.2)then
          alpha=sqrt(2d0)*gf/pi*mw2*sw2
          qgwqcd=1
          qgwbos=0
          qgwferm=0
c one-loop W width without QCD corrections
        elseif(qgw.eq.3)then
          qgwqcd=0
          qgwbos=1
          qgwferm=1
c one-loop W width with QCD corrections
        elseif(qgw.eq.4)then
          qgwqcd=1
          qgwbos=1
          qgwferm=1
        endif
	gw=wwidth()
        alpha=alphahelp
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting options                                             c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c qfast=0: form factors for virtual corr. from full formulas (slow !)
c qfast=1: expansion into Legendre polynomials (fast !) 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qfast=1
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c qsoft=1,2,3: definition of the DPA part of radiative corrections,            
c              terms subtracted from the virtual corrections in DPA     
c              are fixed by:                                    
c qsoft=1: endpoint part as defined in eq. (4.29) of hep-ph/0006307 
c          (default!)    
c qsoft=2: only logarithms as defined in eq. (4.2.53) of 
c          hep-ph/0008033 
c qsoft=3: YFS factor for virtual photons as defined in eq. (4.31) 
c          of hep-ph/0006307
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qsoft=1
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c son=1,2,3,4: choice of on-shell projection 
c              for virtual corrections in DPA
c RacoonWW provides two different versions for the projection 
c of off-shell momenta to on-shell momenta for               
c e^+(p1)+e^-(p2) -> f1(p3)+ fbar2(p4) + f3(p5)+ fbar4(p6):              
c ( CMS of e^+e^- is assumed )
c son = 1: directions of momenta of W^+, f1 and f3 are fixed
c son = 2: directions of momenta of W^+, f2 and f3 are fixed
c son = 3: directions of momenta of W^+, f1 and f4 are fixed
c son = 4: directions of momenta of W^+, f2 and f4 are fixed
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      son=1
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c radiative corrections
c qbos=1 : with virtual bosonic radiative corrections 
c qferm=1: with fermion-loop radiative corrections 
c qnf>0  : non-factorizable corrections are included where
c          qnf=1, 2, or 3 defines how the Coulomb 
c          singularity is implemented.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(src.ne.0)then
        if(qprop.ne.1)then
           write(*,'(a)')'warning for RC:'
           write(*,'(a)')'RacoonWW overwrites your choice for qprop,'
           write(*,'(a)')'since not allowed. See NOTE.4 in README.'
           qprop=1
        endif
        if(ssigepem5.ne.5)then
           write(*,'(a)')'warning for RC:' 
           write(*,'(a)')
     $          'RacoonWW overwrites your choice for ssigepem5,'
           write(*,'(a)')'since not allowed. See NOTE.5 in README.'
           ssigepem5=5
        endif
        if(qnf.eq.0)then
           write(*,'(a)')'warning for RC:' 
           write(*,'(a)')'non-factorizable corrections'
           write(*,'(a)')'should be included. See NOTE.3 in README.'
           qnf=2
        endif
        qborn=0
        qbos=1 
        qferm=1       
        sborn5=1
        sborng5=0
        if (qqcd.eq.1.or.qqcd.eq.3) sborng5=1
        if (qqcd.eq.1) ssigepemg5=1
        if (qqcd.eq.3) ssigepemg5=5
      else
        qborn=0
        qbos=0
        qferm=0
        qnf=0
        if(qqcd.eq.1.or.qqcd.eq.3.or.(sborng5.eq.1.and.qqcd.eq.2))then
           write(*,'(a)')'warning for tree-level:' 
           write(*,'(a)')'RacoonWW overwrites your choice for qqcd,'
           write(*,'(a)')'since not allowed. See NOTE.6 in README.'
           if(sborng5.ne.1)qqcd=2
           if(sborng5.eq.1)qqcd=0
        endif
      endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c partial W widths (calculated with the same options as chosen for
c the calculation of the amplitude):
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qgwborn=1
      if(sborn4.eq.0.and.siba.eq.0.and.sborn5.eq.0)qgwborn=0
      qgwqcd=0
      if(qqcd.ne.0)qgwqcd=1
      qgwbos=qbos
      qgwferm=qferm
c W->l nu_l:
      gwl=pWwidth(-1d0,0d0)
c W->q q':
      gwq=pWwidth(-1d0/3d0,2d0/3d0)
c effective branching ratios:
      brlep=3d0*gwl/gw
      brhad=2d0*gwq/gw
      brtot=(3d0*gwl+2d0*gwq)/gw
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting particle parameters                                 c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c fermion names
      gfname(0,-1,1)='e'
      gfname(0,-1,2)='mu'
      gfname(0,-1,3)='tau' 
      gfname(0,+1,1)='nu_e' 
      gfname(0,+1,2)='nu_mu'
      gfname(0,+1,3)='nu_tau'
      gfname(1,-1,1)='d'
      gfname(1,-1,2)='s'
      gfname(1,-1,3)='b'
      gfname(1,+1,1)='u'
      gfname(1,+1,2)='c'
      gfname(1,+1,3)='t'
      dgfname(+1)='      '
      dgfname(-1)=' anti-'
c beam momenta               
      pbeam(1,0)=0.5d0*energy
      pbeam(1,1)=0d0
      pbeam(1,2)=0d0
      pbeam(1,3)=0.5d0*energy
      pbeam(2,0)=0.5d0*energy
      pbeam(2,1)=0d0
      pbeam(2,2)=0d0
      pbeam(2,3)=-0.5d0*energy
c particle parameters are fixed
      dir(1)=1 
      dir(2)=1
      do i1=3,7
         dir(i1)=-1
      end do
      fname(1)=gfname(0,-1,1)
      fname(2)=gfname(0,-1,1)
      do i1=1,6
        dfname(i1)=dgfname(dir(i1)*(-1)**i1)      
      do i2=0,1
      do i3=-1,+1,2
      do i4=1,3
        if(fname(i1).eq.gfname(i2,i3,i4))then
          quark(i1)=i2
          iso(i1)=i3
          gen(i1)=i4
        endif
      enddo
      enddo
      enddo
      if(iso(i1).eq.0)then
        write(*,'(a)')' Wrong particle name: ',fname(i1)
        stop
      endif
      enddo
       process=dfname(1)//fname(1)//dfname(2)//fname(2)//' -> '//
     *   dfname(3)//fname(3)//dfname(4)//fname(4)//
     *   dfname(5)//fname(5)//dfname(6)//fname(6)//' (+gamma)'
       do i1=88,1,-1
         if(process(i1:i1+1).eq.'  ')then
           process=process(1:i1)//process(i1+2:90)//' '
         endif
       enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     setting cut parameters                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c ADLO/TH cuts
      if(scuts.eq.1)then   
        ecutp=0.1d0            ! photon energy cut
        ecutl=1d0              ! charged-lepton energy cut
        ecutq=3d0              ! quark energy cut
        scutqq=5d0             ! quark-quark invariant mass cut
        ccutpb=1d0             ! angular cut between photon and beam
        ccutpl=5d0             ! angular cut between photon and charged lepton
        ccutpq=5d0             ! angular cut between photon(gluon) and quark
        ccutll=5d0             ! angular cut between charged leptons 
        ccutqq=0d0             ! angular cut between quarks 
        ccutlq=5d0             ! angular cut between charged lepton and quark
        ccutlb=10d0            ! angular cut between charged lepton and beam 
        ccutqb=0d0             ! angular cut between quark and beam  
c NLC/TH cuts
      elseif(scuts.eq.2)then
        ecutp=0.1d0            ! photon(gluon) energy cut
        ecutl=1d0              ! charged-lepton energy cut
        ecutq=3d0              ! quark energy cut
        scutqq=0.1d0           ! quark-quark invariant mass cut
        ccutpb=1d0             ! angular cut between photon and beam
        ccutpl=5d0             ! angular cut between photon and charged lepton
        ccutpq=5d0             ! angular cut between photon(gluon) and quark
        ccutll=5d0             ! angular cut between charged leptons
        ccutqq=0d0             ! angular cut between quarks
        ccutlq=5d0             ! angular cut between charged lepton and quark
        ccutlb=10d0            ! angular cut between charged lepton and beam
        ccutqb=5d0             ! angular cut between quark and beam
      endif
c filling cut parameters
      if(scuts.eq.1.or.scuts.eq.2.or.scuts.eq.10)then     
        ecut(7)=ecutp
        ccut(1,7)=ccutpb
        ccut(2,7)=ccutpb
        do i1=3,6
          if(quark(i1).eq.0.and.iso(i1).lt.0)then     ! charged leptons
            ecut(i1)=ecutl
            ccut(1,i1)=ccutlb
            ccut(2,i1)=ccut(1,i1)
            ccut(i1,7)=ccutpl
          elseif(quark(i1).eq.1)then                  ! quarks
            ecut(i1)=ecutq
            ccut(1,i1)=ccutqb
            ccut(2,i1)=ccut(1,i1)
            ccut(i1,7)=ccutpq
          endif
        do i2=i1+1,6
          if(quark(i1).eq.1.and.quark(i2).eq.1)then
            scut(i1,i2)=scutqq
            ccut(i1,i2)=ccutqq
          elseif(quark(i1).eq.0.and.iso(i1).lt.0.and.
     *           quark(i2).eq.0.and.iso(i2).lt.0)then
            ccut(i1,i2)=ccutll
          elseif((quark(i1).eq.0.and.iso(i1).lt.0.and.quark(i2).eq.1)
     *       .or.(quark(i1).eq.1.and.quark(i2).eq.0.and.iso(i2).lt.0)
     *          )then
            ccut(i1,i2)=ccutlq
          endif          
        enddo
        enddo
      endif
c calculating cut parameters
c angle -> cos(angle)
      do i1=1,7
      do i2=1,7
        ccut(i1,i2)=dcos(ccut(i1,i2)*pi/180d0)
      enddo
      enddo
c mass -> mass squared
      do i1=1,7
      do i2=1,7
        scut(i1,i2)=scut(i1,i2)**2
      enddo
      enddo
c filling whole common block
      do i1=1,7
      do i2=i1+1,7
        ccut(i1,i2)=min(ccut(i1,i2),ccut(i2,i1))
        scut(i1,i2)=max(scut(i1,i2),scut(i2,i1))
      enddo
      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     initialize subroutines                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc 
c initializing amplitude
      call initparam
      call init_epem
      if(sborn4.eq.2.or.sborn4.eq.3)then
        if((nc.eq.1.and.cc.eq.0).or.
     $        (nc.eq.1.and.cc.eq.1.and.(sym.eq.1.or.sym.eq.2)))then
          write(*,'(a)')' Not a valid final-state for DPA Born!'
          stop
        endif
      endif
      if(sborng5.ne.0)then
        if((nc.eq.1.and.cc.eq.0).or.
     $    (nc.eq.1.and.cc.eq.1.and.sym.ne.0).or.
     $    (quark(3).eq.0.and.quark(4).eq.0.and.quark(5).eq.0.and.
     $    quark(6).eq.0))then
          if(src.eq.0)
     $      write(*,'(a)')' Not a valid final-state for 4f+gluon!'
          if(src.ne.0)
     $      write(*,'(a)')' Not a valid final-state for qqcd=1 or 3!'
          stop
        endif
      endif
c initializing virtual corrections
      if (src.ne.0.or.siba.ne.0.or.sborn4.ge.2) then
        if(cc.eq.1.and.nc.eq.0)then
          if(quark(3).eq.0.and.quark(5).eq.0)i1=1
          if(quark(3).eq.0.and.quark(5).eq.1)i1=2
          if(quark(3).eq.1.and.quark(5).eq.0)i1=3
          if(quark(3).eq.1.and.quark(5).eq.1)i1=4
        elseif(cc.eq.1.and.nc.eq.1.and.(sym.eq.0.or.sym.eq.3))then
          if(quark(3).eq.0)i1=1
          if(quark(3).eq.1)i1=4
        elseif(src.ne.0)then
          write(*,'(a)')
     *         ' Not a valid final-state for virtual corrections!'
          stop
        endif
        call init_4f(i1,gen(3),gen(5))
c calculate renormalization constants
        call calcrenc
        call calcrenc_bgf
        call calcrenc_ferm
      end if
c initialize expansion of formfactors in Legendre polynomials
      if(qfast.eq.1.and.siba.eq.0.and.src.ne.0)then
        energy2=energy**2
        ng=20 
        nl=2*ng-1
        call Cfourier(energy2,nl,ng)
        qfast=1
      endif
c naive QCD factor:
      nqcdfac=1d0
      if(qqcd.eq.2)then
        do i1=3,6
          if(quark(i1).ne.0)nqcdfac=nqcdfac*dsqrt(1d0+alphas/pi)
        enddo
      endif
c initializing phase-space slicing
      write(nout,*)' '
      if(smc.ne.2)then
        write(nout,'(a10,i2,(a))')
     $        'smc=',smc,': Phase-space-slicing branch of RacoonWW' 
        write(nout,'(14x,(a))')'======================================'
        write(nout,'(a)')' '
        write(30,'(a)')' '
        write(30,'(a)')' Phase-space-slicing branch of RacoonWW'
        write(30,'(a)')' ======================================'
        write(30,'(a)')' '
        call initslicing(energy,conv,scale,nevents,nout)
      endif
c initializing subtraction method
      if(smc.eq.2)then
        write(nout,'(a10,i2,(a))')
     $        'smc=',smc,': Subtraction-method branch of RacoonWW'
        write(nout,'(14x,(a))')'======================================'
        write(30,'(a)')' '
        write(30,'(a)')' Subtraction-method branch of RacoonWW'
        write(30,'(a)')' ====================================='
        write(30,'(a)')' '
        call initsubtraction(energy,conv,scale,nout)
      endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c switch for PAW/HBOOK:
c qpaw=1: PAW/HBOOK is used for generating histograms 
c         in addition to default histogram routine; 
c         the output is written in pawplot.paw !
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      qpaw=0
c      if(qpaw.eq.1)call setup_graphs(smc,energy)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     output                                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c input parameters
 998  format(/,' Input parameters:',/,' -----------------',/,
     *     4x,'CMF energy = ',f9.5,' GeV',',',4x,
     $     'Number of events = ',i10,',',/,/,
     *     2x,'alpha(0) = 1/', f12.7,',',
     *     1x,'alpha(MZ) = 1/',f9.5,',',
     *     3x,'alpha_s = ',f7.5,',',/,
     *     8x,'GF = ',g12.7,',',/,
     *     8x,'MW = ',f9.5,',',
     *     13x,'MZ = ',f9.5,',',
     *     10x,'MH = ',f9.5,',',/,
     *     8x,'GW = ',f9.5,',',
     *     13x,'GZ = ',f9.5,',',/,
     *     8x,'me = ',g13.8,',',8x,'mmu = ',f13.9,',',
     *     4x,'mtau = ',f9.5,',',/,
     *     8x,'mu = ',f9.5,',',13x,'mc = ',f9.5,',',
     *     10x,'mt = ',f9.5,',',/,
     *     8x,'md = ',f9.5,',',13x,'ms = ',f9.5,',',
     *     10x,'mb = ',f9.5,'.',/)      
      write(nout,998) energy,nevents,
     *     1d0/alpha0,1d0/alphaz,alphas,GF,
     *     mw,mz,mh,gw,gz,ml(1),ml(2),ml(3),
     *     mqp(1),mqp(2),mqp(3),mqm(1),mqm(2),mqm(3)
      write(nout,'(a)')' Effective branching ratios: '
 999  format(' leptonic BR =',f9.5,',',
     $     ' hadronic BR =',f9.5,',',
     $     ' total BR =',f9.5)
      write(nout,999)brlep,brhad,brtot
      write(nout,*)' '
      write(nout,'(" Process: ",a60)')process
      write(nout,*)' '
c input-parameter scheme
      if(qalp.eq.0)then
        write(nout,'(a10,i2,(a))')
     $        'qalp=',qalp,': alpha(0)-parametrization scheme.'
      elseif(qalp.eq.1)then
        write(nout,'(a10,i2,(a))')
     $        'qalp=',qalp,': alpha(MZ)-parametrization scheme.'
      elseif(qalp.eq.2)then
        write(nout,'(a10,i2,(a))')
     $        'qalp=',qalp,': GF-parametrization scheme.'
      endif
c calculation of W width
      if(qgw.eq.1)then
        write(nout,'(a10,i2,(a))')
     $        'qgw=',qgw,': tree-level W-boson width calculated.'
      elseif(qgw.eq.2)then
        write(nout,'(a10,i2,(a))')
     $        'qgw=',qgw,': IBA W-boson width calculated.'
      elseif(qgw.eq.3)then
        write(nout,'(a10,i2,(a))')
     *  'qgw=',qgw,
     $     ': one-loop W-boson width calculated (without QCD corr.).'
      elseif(qgw.eq.4)then
        write(nout,'(a10,i2,(a))')
     *    'qgw=',qgw,
     $        ': one-loop W-boson width calculated (with QCD corr.).'
      endif
c treatment of W width in propagator
      if(qprop.eq.0)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,': width = 0.'
      elseif(qprop.eq.1)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,': constant width.'
      elseif(qprop.eq.2)then
        write(nout,'(a10,i2,(a))')
     $        'qprop=',qprop,': constant width * theta(p^2).'
      elseif(qprop.eq.3)then
        write(nout,'(a10,i2,(a))')'qprop=',qprop,': running  width.'
      elseif(qprop.eq.4)then
        write(nout,'(a10,i2,(a))')
     $        'qprop=',qprop,': constant width and complex sw,cw.'
      endif
c improved born approximation
      if(siba.eq.1)then
        write(nout,'(a10,i2,(a))')
     $        'siba=',siba,': improved Born approximation.'
      endif
c tree-level processes ee -> 4f
      if(sborn4.eq.0.and.siba.eq.0.and.sborn5.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')
     $       'sborn4=',sborn4,
     $       ': tree-level process ee -> 4f NOT included.'
      elseif(sborn4.ne.0)then
        write(nout,*)' '
        if(sborn4.eq.1)write(nout,'(a10,i2,(a))')
     $     'sborn4=',sborn4,': tree-level process ee -> 4f.'
        if(sborn4.eq.2)write(nout,'(a10,i2,(a))')
     $     'sborn4=',sborn4,': tree-level ee -> 4f in DPA'//
     $       ' (off-shell phase space)'
        if(sborn4.eq.3)write(nout,'(a10,i2,(a))')
     $     'sborn4=',sborn4,': tree-level ee -> 4f in DPA.'
        if(sqcdepem.eq.1)then
          write(nout,'(a10,i2,(a))')
     $       'sqcdepem=',sqcdepem,
     $          ': gluon-exchange diagrams included.'
        endif
        if(ssigepem4.eq.0)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem4=',ssigepem4,
     $          ': all electroweak diagrams included.'
        elseif(ssigepem4.eq.1)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem4=',ssigepem4,
     $          ': only electroweak WW-signal diagrams.'
        elseif(ssigepem4.eq.2)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem4=',ssigepem4,
     $          ': only electroweak ZZ-signal diagrams.'
        elseif(ssigepem4.eq.3)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem4=',ssigepem4,
     *          ': only electroweak ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem4.eq.4)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem4=',ssigepem4,
     *          ': only electroweak WW,ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem4.eq.5)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem4=',ssigepem4,
     $          ': only diagrams of CC11 class.'
        endif
      endif
c tree-level processes ee -> 4fa
      if(sborn5.ne.0.and.src.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')
     $       'sborn5=',sborn5,': tree-level process ee -> 4f+gamma.'
        if(sqcdepem.eq.1)then
          write(nout,'(a10,i2,(a))')
     $          'sqcdepem=',sqcdepem,
     $          ': gluon-exchange diagrams included.'
        elseif(sqcdepem.eq.2)then
          write(nout,'(a10,i2,(a))')
     $          'sqcdepem=',sqcdepem,
     $          ': only gluon-exchange diagrams.'
        endif
        if(ssigepem5.eq.0)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem5=',ssigepem5,
     $          ': all electroweak diagrams included.'
        elseif(ssigepem5.eq.1)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem5=',ssigepem5,
     $          ': only electroweak WW-signal diagrams.'
        elseif(ssigepem5.eq.2)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem5=',ssigepem5,
     $          ': only electroweak ZZ-signal diagrams.'
        elseif(ssigepem5.eq.3)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem5=',ssigepem5,
     *      ': only electroweak ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem5.eq.4)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem5=',ssigepem5,
     *      ': only electroweak WW,ZZ,ZA,AA-signal diagrams.'
        elseif(ssigepem5.eq.5)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepem5=',ssigepem5,
     $          ': only diagrams of CC11 class.'
        endif
      endif
c tree-level processes ee -> 4fg
      if(sborng5.ne.0.and.src.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')
     $       'sborng5=',sborng5,': tree-level process ee -> 4f+gluon.'
        if(ssigepemg5.eq.1)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepemg5=',ssigepemg5,
     $          ': only electroweak WW-signal diagrams.'
        elseif(ssigepemg5.eq.5)then
          write(nout,'(a10,i2,(a))')
     $          'ssigepemg5=',ssigepemg5,
     $          ': only diagrams of CC11 class.'
        endif
      endif
c radiative-correction mode:
      if(src.ne.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'src=',src,
     $    ': virtual corrections in DPA and real corrections included.'
        write(nout,'(a10,i2,(a))')'ssigepem5=',ssigepem5, 
     $    ': real photon corr. : only CC11 class of diagrams included.'
        if(qqcd.eq.1)then
          write(nout,'(a10,i2,(a))')
     $       'qqcd=',qqcd,
     $       ': QCD corrections to CC03 included (full matrix element)'
        else if(qqcd.eq.2)then
          write(nout,'(a10,i2,(a))')
     *          'qqcd=',qqcd,': naive QCD corrections included '
        else if(qqcd.eq.3)then
          write(nout,'(a10,i2,(a))')
     $       'qqcd=',qqcd,
     $       ': QCD corrections to CC11 included (full matrix element)'
        endif
        if(qreal.eq.1)then
          write(nout,'(a10,i2,(a))')'qreal=',qreal,
     *      ': imaginary part of virtual corrections neglected.'
        else
          write(nout,'(a10,i2,(a))')'qreal=',qreal,
     *      ': imaginary part of virtual corrections included.'
        endif
        if(qnf.eq.1)then
          write(nout,'(a10,i2,(a))')'qnf=',qnf,
     $          ': on-shell Coulomb singularity included.'
        elseif(qnf.eq.2)then
          write(nout,'(a10,i2,(a))')'qnf=',qnf,
     $          ': off-shell Coulomb singularity included.'
        elseif(qnf.eq.3)then
          write(nout,'(a10,i2,(a))')'qnf=',qnf,
     $          ': off-shell Coulomb singularity '//
     *          'with off-shell Born included.'
        endif 
      endif
c higher-order ISR
      if(sisr.eq.0)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'sisr=',sisr,
     *       ': higher-order initial-state radiation NOT included.'
      else
        if(smc.ne.3) 
     $        write(nout,'(a10,i2,(a))')'sisr=',sisr,
     *        ': initial-state radiation up to order alpha^3 included.'
        if(smc.eq.3) 
     $        write(nout,'(a10,i2,(a))')'sisr=',sisr,
     *        ': initial-state radiation up to order alpha^2 included.'
      endif      
c recombination cuts
      if(srecomb.eq.1.or.srecomb.eq.2.or.srecomb.eq.3)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'srecomb=',srecomb,
     $       ': with photon recombination:'
        do i2=1,4
          write(nout,'(" precomb(",i1,")"," = ",f9.5)')i2,precomb(i2)
        enddo
      endif
      if(srecombg.eq.1.or.srecombg.eq.2)then
        write(nout,*)' '
        write(nout,'(a10,i2,(a))')'srecombg=',srecombg,
     $       ': with gluon recombination:'
        do i2=1,2
          write(nout,'(" precombg(",i1,")"," = ",f9.5)')i2,precombg(i2)
        enddo
      endif
c separation cuts
      write(nout,*)' '
      if(scuts.eq.0)then
        write(nout,'(a10,i2,(a))')'scuts=',scuts,
     $        ': no separation cuts !'
      else
        write(nout,'(a10,i2,(a))')'scuts=',scuts,
     $        ': with separation cuts:'
        do i1=1,7
          if(ecut(i1).gt.0d0)then
            write(nout,'(" energy(",i1,")  > ",f9.5," GeV")')
     *        i1,ecut(i1)
          endif
        enddo
        do i1=1,7
        do i2=i1+1,7 
          if(ccut(i1,i2).lt.1d0)then
            write(nout,'(" angle(",i1,",",i1,") > ",f9.5," deg")')
     *        i1,i2,dacos(ccut(i1,i2))/pi*180d0
          endif
        enddo
        enddo
        do i1=1,7
        do i2=i1+1,7 
          if(scut(i1,i2).gt.0d0)then
            write(nout,'(" mass(",i1,",",i1,")  > ",f9.5," GeV")')
     *        i1,i2,dsqrt(scut(i1,i2))
          endif
        enddo
        enddo
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     numerical integration                                       c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c initialization:
      steps=1000000
      if(smc.ne.2)nsubevent=8
      if(smc.eq.2)nsubevent=maxe
c initialization of weights and four-momenta:
      do i1=1,maxe
        weight(i1)=0d0
        weightmax(i1)=0d0
        do i2=1,7
        do i3=0,3   
          p(i2,i3,i1)=0d0
        enddo
        enddo
      enddo
      weightborn=0d0
      do i1=1,7
      do i2=0,3   
        pborn(i1,i2)=0d0
      enddo
      enddo
      do i1=1,7
      do i2=0,3   
        pu(i1,i2)=0d0
      enddo
      enddo
      weighttotmax=0d0
      nunweighted(-1)=0
      nunweighted(1)=0
      novershoot=0
      averagemax=0d0
      average=0d0
      sigma=0d0
      avb=0d0
      sigb=0d0
      call settings(1,pbeam,p(1,0,1),0d0,energy,q,1,1)
      if(nevents.ge.steps)then
        write(nout,*)' '
        write(nout,'(a)')'    events  :  intermediary results   :'//
     *    '  preliminary results '
      endif
c start of MC integration
      do n=1,nevents
c generation of random numbers
        do i1=1,15
          call rans(rand)
          random(i1)=rand
        enddo
c calls for MC kernel
c sunw: switch for unweighting procedure
        sunw=1
        if(smc.eq.1)then
          y=kern(random,1d0/nevents)
        elseif(smc.eq.2)then
          call subtraction(random)
        elseif(smc.eq.3.and.sunw.eq.0)then
          y=kern(random,1d0/nevents) 
        elseif(smc.eq.3.and.sunw.eq.1)then
c start of unweighting procedure
          ninit=10000
          if(n.le.ninit)then
            y=kern(random,1d0/nevents)
c determine overall maximal weight and 
c maximal weight for each sub-contribution
            do i1=1,nsubevent
              if(weightmax(i1).lt.weight(i1))weightmax(i1)=weight(i1)
              weighttotmax=max(weighttotmax,weightmax(i1))
            enddo
            if(n.eq.ninit)then
              do i1=1,nsubevent
                if(weightmax(i1).ne.0d0)then
                  nweight(i1)=int(weighttotmax/weightmax(i1))
                else
                  nweight(i1)=0
                endif
              enddo          
              weighttotmax=2d0*weighttotmax
            endif    
          else
            do i1=1,nsubevent
              if(nweight(i1).ne.0)then 
                if(mod(dble(n),dble(nweight(i1))).eq.0d0)then
                  wflag(i1)=1
                else
                  wflag(i1)=0
                endif
              end if
            enddo 
            y=kern(random,1d0/nevents)
            call rans(rand)
            weightborn=weightborn*dble(nweight(1)*wflag(1))
            do i1=1,nsubevent
              weight(i1)=weight(i1)*dble(nweight(i1)*wflag(i1))
              if(nweight(i1).ne.0)then
                if(mod(dble(n),dble(nweight(i1))).eq.0d0)then
                  if(weight(i1).gt.weighttotmax)then
c events with weight > weightmax
                    novershoot=novershoot+1
                    write(nout,'(" Warning: weight>weightmax ",2i12)')
     *                   novershoot,n
                  endif
                  if(rand*weighttotmax.lt.dabs(weight(i1)))then
                    if(weight(i1).gt.0d0)then
                      nunweighted(1)=nunweighted(1)+1
                      do i2=0,3
                      do i3=1,2
                        pu(i3,i2)=pbeam(i3,i2)
                      enddo
                      do i3=3,7
                        pu(i3,i2)=p(i3,i2,i1)
                      enddo
                      enddo
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                     c
c Unweighted events                                                   c
c                                                                     c
c The four-momenta of the external particles can be read out here !   c 
c beam (e+)                      pu(1,0:3)                            c
c beam (e-)                      pu(2,0:3)                            c
c final-state fermion 3          pu(3,0:3)                            c
c final-state anti-fermion 4     pu(4,0:3)                            c
c final-state fermion 5          pu(5,0:3)                            c
c final-state anti-fermion 6     pu(6,0:3)                            c
c photon                         pu(7,0:3)                            c
c                                                                     c
c Remark:                                                             c
c CC-processes: W+ -> f3 + f4, W- -> f5 + f6                          c
c MC-processes: W+ -> f3 + f6, W- -> f4 + f5                          c
c                                                                     c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
                    else
c "unweighted" events with weight=-1
                      nunweighted(-1)=nunweighted(-1)+1
                      write(nout,'(" Warning: weight=-1 ",2i12)')
     *                     nunweighted(-1),n
                    endif
                  endif
                endif
              endif
            enddo
          endif
        endif
c end of unweighting procedure
c summation of sub-contributions and filling of histograms
        y=0d0
        do i1=1,nsubevent
          y=y+weight(i1)*nqcdfac
          call settings(i1,pbeam,p(1,0,i1),weight(i1)*nqcdfac,
     $         energy,q,n,2)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c When using paw for histogram plotting, please
c uncomment the following lines:
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c          if(qpaw.eq.1.and.weight(i1).ne.0d0)then
c            call graphs(smc,i1,q,weight(i1)*nqcdfac,1d0/nevents)
c            if(smc.le.2.and.i1.ne.1)then
c              if((smc.eq.2.and.i1.le.5).or.(smc.eq.1.and.i1.le.6))  
c     $          call graphs_k(smc,i1,weight(i1)*nqcdfac,1d0/nevents)
c            endif
c            if(smc.eq.3.and.i1.le.6)then
c              if(i1.eq.1)then  
c     $          call graphs_k(smc,i1,
c     $              (weight(i1)-weightborn)*nqcdfac,1d0/nevents)
c              else
c                call graphs_k(smc,i1,weight(i1)*nqcdfac,1d0/nevents)
c              endif
c            endif
c            if((smc.eq.2.and.i1.gt.5).or.(smc.ne.2.and.i1.gt.6)) 
c     $        call graphs_r(smc,i1,weight(i1)*nqcdfac,1d0/nevents)
c          endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        enddo
c        if(qpaw.eq.1.and.weightborn.ne.0d0)
c     $       call graphs_b(smc,1,weightborn,1d0/nevents)
        average=average+y
        sigma=sigma+y**2
c Born
        avb=avb+weightborn
        sigb=sigb+weightborn**2
c preliminary results
        av=av+y
        si=si+y**2
        if(mod(n,steps).eq.0)then
          write(nout,'(i10,2("  : ",f10.5," +-",f9.5))')n,
     *      av/dble(steps),
     *      dsqrt(dabs(si-av**2/dble(steps)))/dble(steps),
     *      average/dble(n),
     *      dsqrt(dabs(sigma-average**2/dble(n)))/dble(n)
          av=0d0
          si=0d0
        endif
c maximal weight
        if(y.gt.averagemax)then
          averagemax=y
        endif
        call settings(1,pbeam,pborn,weightborn,energy,q,n,3)
      enddo 
c end of MC integration
      call settings(1,pbeam,p(1,0,1),0d0,energy,q,nevents,4)
c calculation of total cross section and statistical error
      average=average/dble(nevents)
      sigma=sigma/dble(nevents)**2
      sigma=dsqrt(dabs(sigma-average**2/dble(nevents)))
      averagemax=averagemax/dble(nevents)
c Born
      avb=avb/dble(nevents)
      sigb=sigb/dble(nevents)**2
      sigb=dsqrt(dabs(sigb-avb**2/dble(nevents)))
      if(smc.ne.2)then
c calculation of errors for subcontributions:
        do i1=1,maxe
          dscinti(i1)=dsqrt(dabs(dconti_tot(i1)-
     $         conti_tot(i1)**2/dble(nevents)))
        enddo
      elseif(smc.eq.2)then
        do i1=1,4
          subav(i1)=subav(i1)/dble(nevents)
          subsi(i1)=subsi(i1)/dble(nevents)**2
          subsi(i1)=dsqrt(dabs(subsi(i1)-subav(i1)**2/dble(nevents)))
          submax(i1)=submax(i1)/dble(nevents)
        enddo
      endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     output                                                      c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
 1001 format((a),f4.1,"%")
 1002 format(a22,f16.10," fb")
 1003 format(" (event = ",i9,", channel = ",i3,")")
 1004 format(" % events rejected in calculation of weight(",i2,"):"
     $     ,f4.1,"%")
      write(nout,'(a)')' '
      write(nout,'(a)')' Result:'
      write(nout,'(a)')' ------- '
      write(nout,'(" Number of events   = ",i16)')nevents
      write(nout,1002)' Average            = ',average
      write(nout,1002)' Standard deviation = ',sigma
      write(nout,1002)' Maximal weight     = ',averagemax
      write(nout,'(a)')' '
      if(smc.eq.3.and.sunw.eq.1)then
         write(nout,'(a)')' Tree-level four-fermion cross section:'
         write(nout,1002)' Average            = ',avb
         write(nout,1002)' Standard deviation = ',sigb
         write(nout,'(a)')' '
      endif
c additional output for slicing branch
      if(smc.eq.1.or.(smc.eq.3.and.sunw.eq.0))then
         if(qqcd.eq.2)then 
           write(nout,'(a)')' Subcontributions without naive QCD:'
           write(nout,'(a)')' -----------------------------------'
         else
           write(nout,'(a)')' Subcontributions :'
           write(nout,'(a)')' ------------------'
         endif
c write subcontributions 
         write(nout,'(a)')' Tree-level four-fermion cross section:'
         write(nout,1002)' Average            = ',conti_tot(1)
         write(nout,1002)' Standard deviation = ',dscinti(1)
         write(nout,1002)' Maximal weight     = ',wmax(1)/dble(nevents)
         write(nout,*)' '
         write(nout,'(a)')' 2->4 DPA part :'
         write(nout,1002)' Average            = ',conti_tot(2)
         write(nout,1002)' Standard deviation = ',dscinti(2)
         write(nout,1002)' Maximal weight     = ',wmax(2)/dble(nevents)
         write(nout,*)' '
         write(nout,'(a)')
     $        ' 2->4 virtual+soft+final-state collinear photon part:'
         if(sisr.ne.0)then
            if(smc.ne.3)write(nout,'(a)')' without LL O(alpha) ISR'
            if(smc.eq.3)write(nout,'(a)')' with LL O(alpha^2) ISR'
         endif
         write(nout,1002)' Average            = ',conti_tot(3)
         write(nout,1002)' Standard deviation = ',dscinti(3)
         write(nout,1002)' Maximal weight     = ',wmax(3)/dble(nevents)
         write(nout,*)' '
         write(nout,'(a)')
     $        ' 2->4 initial-state collinear photon part (e+):'
         if(sisr.ne.0)then
            if(smc.ne.3)write(nout,'(a)')' without LL O(alpha) ISR'
            if(smc.eq.3)write(nout,'(a)')' with LL O(alpha^2) ISR'
         endif
         write(nout,1002)' Average            = ',conti_tot(4)
         write(nout,1002)' Standard deviation = ',dscinti(4)
         write(nout,1002)' Maximal weight     = ',wmax(4)/dble(nevents)
         write(nout,*)' '
         write(nout,'(a)')
     $        ' 2->4 initial-state collinear photon part (e-):'
         if(sisr.ne.0)then
            if(smc.ne.3)write(nout,'(a)')' without LL O(alpha) ISR'
            if(smc.eq.3)write(nout,'(a)')' with LL O(alpha^2) ISR'
         endif
         write(nout,1002)' Average            = ',conti_tot(5)
         write(nout,1002)' Standard deviation = ',dscinti(5)
         write(nout,1002)' Maximal weight     = ',wmax(5)/dble(nevents)
         write(nout,*)' '
         if(siba.eq.0)then 
            if(smc.ne.3)write(nout,'(a)')
     $           ' 2->4 LL initial-state radiation up to O(alpha^3):'
            if(smc.eq.3)write(nout,'(a)')
     $           ' 2->4 LL O(alpha^2) initial-state radiation:'
         else
           write(nout,'(a)')' Improved Born Approximation:'
         endif
         write(nout,1002)' Average            = ',conti_tot(6)
         write(nout,1002)' Standard deviation = ',dscinti(6)
         write(nout,1002)' Maximal weight     = ',wmax(6)/dble(nevents)
         write(nout,*)' '
         if(src.ne.0)then
            write(nout,'(a)')' 2->5 Bremsstrahlung contribution (QED):'
         else 
            write(nout,'(a)')' Tree-level process e+ e- -> 4f + gamma:'
         endif
         write(nout,1002)' Average            = ',conti_tot(7)
         write(nout,1002)' Standard deviation = ',dscinti(7)
         write(nout,1002)' Maximal weight     = ',wgmax/dble(nevents)
         write(nout,*)' '
         if(src.ne.0)then
            write(nout,'(a)')' 2->5 Bremsstrahlung contribution (QCD):'
         else 
            write(nout,'(a)')' Tree-level process e+ e- -> 4f + gluon:'
         endif
         write(nout,1002)' Average            = ',conti_tot(8)
         write(nout,1002)' Standard deviation = ',dscinti(8)
         write(nout,1002)' Maximal weight     = ',wqmax/dble(nevents)
         rej(1)=100d0*real(ncut1_k(2))/real(nevents)
         rej(2)=100d0*real(ncut2_k(2))/real(nevents)
         rej(3)=100d0*real(ncut3_k(2))/real(nevents)
         rej(4)=100d0*real(ncut4_k(2))/real(nevents)
         rej(5)=100d0*real(ncut5_k(2))/real(nevents)
         rej(6)=100d0*real(ncut6_k(2))/real(nevents)
         rej(7)=100d0*real(ncut_r(2))/real(nevents)
         rej(8)=100d0*real(ncutq_r(2))/real(nevents)
         write(nout,*)' '
         do i1=1,nsubevent
           write(nout,1004)i1,rej(i1)
         enddo
         write(nout,*) ' '
      endif
c additional output for subtraction branch
      if(smc.eq.2)then
        if(qqcd.eq.2)then 
          write(nout,'(a)')' Subcontributions without naive QCD:'
          write(nout,'(a)')' -----------------------------------'
        else
          write(nout,'(a)')' Subcontributions :'
          write(nout,'(a)')' ------------------'
        endif
        write(nout,'(a)')' Tree-level four-fermion cross section:'
        write(nout,1002)' Average            = ',subav(1)
        write(nout,1002)' Standard deviation = ',subsi(1)
        write(nout,1002)' Maximal weight     = ',submax(1)
        write(nout,1003)subn(1),subch(1)
        write(nout,*)' '
        if(sisr.ne.0)then
          write(nout,'(a)')' 2->4 part of radiative corrections'//
     *      ' without O(alpha) LL ISR:'
        else
          write(nout,'(a)')' 2->4 part of radiative corrections:'
        endif
        write(nout,1002)' Average            = ',subav(2)
        write(nout,1002)' Standard deviation = ',subsi(2)
        write(nout,1002)' Maximal weight     = ',submax(2)
        write(nout,1003)subn(2),subch(2)
        write(nout,*)' '
        if(siba.eq.0)then
          write(nout,'(a)')
     *      ' Initial-state radiation up to O(alpha^3):'
        else
          write(nout,'(a)')' Improved Born Approximation:'
        endif
        write(nout,1002)' Average            = ',subav(3)
        write(nout,1002)' Standard deviation = ',subsi(3)
        write(nout,1002)' Maximal weight     = ',submax(3)
        write(nout,1003)subn(3),subch(3)
        write(nout,*)' '
        write(nout,'(a)')' 2->5 part of radiative corrections:'
        write(nout,1002)' Average            = ',subav(4)
        write(nout,1002)' Standard deviation = ',subsi(4)
        write(nout,1002)' Maximal weight     = ',submax(4)
        write(nout,1003)subn(4),subch(4)
        write(nout,*)' '
        do i1=1,nsubevent
          rej(i1)=100d0*dble(subrej(i1))/dble(nevents)
          write(nout,1004)i1,rej(i1)
        enddo
        write(nout,*)' '
        call outputphasespace(30)
        call outputadaptive(30)
      endif
c additional output for unweighted event generator
      if(smc.eq.3.and.sunw.eq.1)then
        write(nout,'(a)')' Number of events'
        write(nout,'(a)')' ----------------'
        write(nout,'(" Unweighted events            = ",i9)')
     *    nunweighted(1)
        write(nout,'(" Events with weight=-1        = ",i9)')
     *    nunweighted(-1)
        write(nout,'(" Events with weight>weightmax = ",i9)')
     *    novershoot
      endif
c setup of paw
c      if(qpaw.eq.1)call setup_paw
      close(30)
c close output file
      if(outputfile(1:2).ne.' ')close(nout)	
      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c                                                                 c
c     pseudo-random-number generator                              c
c                                                                 c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine rans(ran)
      implicit none 
c local variables
      real*8 s1,s2,s3,ran
      integer init
      save init,s1,s2,s3
      data init/0/,s1/0d0/,s2/0d0/,s3/0d0/
      if (init.eq.0) then
        init=1
        s1=dsqrt(2d0)-1d0
        s2=dsqrt(3d0)-1d0
        s3=dsqrt(5d0)-2d0
      endif
      s1=dmod(s1+s2+s3,1d0)
      s2=dmod(s1+s2+s3,1d0)
      s3=dmod(s1+s2+s3,1d0)
      ran=s1
      end  

